/*
 * Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
 */

// This is a manifest file that'll be compiled into including all the files listed below.
// Add new JavaScript/Coffee code in separate files in this directory and they'll automatically
// be included in the compiled file accessible from http://example.com/assets/application.js
// It's not advisable to add code directly here, but if you do, it'll appear at the bottom of the
// the compiled file.
//
//= require console
//= require jquery
//= require jquery_ujs
//= require jquery-ui
//= require jquery.caret
//= require jquery.sendkeys
//= require jquery.pagevisibility.min
//= require bililiteRange
//= require highlight.pack
//= require excanvas
//= require jit
//= require association_trace
//= require_self
//= require source_type_drop_down
//= require activity_filter/activity_filter
//= require work_items

/**
 * Defines methods for handling common events.
 * @constructor
 */
function EventHandlers() {
  $('body').on('click', '.icons-info', function(eventObject) {
    $(this).parents('.info-container').find('.help-info').fadeToggle();
  });
}

/**
 * Turn off links on right side of details
 */
$('.right-side a.activity').click(function(e) {
    e.preventDefault();
});

/**
 * Adds an event handler for ajax errors.
 */
EventHandlers.prototype.addAjaxErrorListener = function() {
  $("body").on("ajax:error", this.handleAjaxError);
};

/**
 * Called when toggling the active status of the build server fails.
 *
 * @param event the event that caused this ajax error
 * @param data the data associated with the event
 * @param status {string} the response status
 * @param jqxhr {jqXHR} the jQuery object that handled the request
 */
EventHandlers.prototype.handleAjaxError = function(event, data, status, jqxhr) {
  var errorMessage = $("#errors-ajax-general").text();
  var errorDiv = $('<div class="message error"></div>').text(errorMessage);
  $(".messages").html(errorDiv);
};

/**
 * Sets the working indicator on/off.
 *
 * @param {Boolean} done true if the working indicator should be on
 */
function pageDone(done) {
  $('.working').toggleClass('done', done);
}

/**
 * selects all img.ctf-avatar and sets
 * load listeners to reveal when successful.
 */
function setAvatars() {
  $('img.ctf-avatar').each(showCtfAvatarIfAvailable);
}

/**
 * Reveals a CTF avatar (presumed to be $(this)) when it loads successfully,
 * except where the associated default avatar is hidden by css
 */
function showCtfAvatarIfAvailable() {
  var ctfAvatar = $(this);
  var defaultAvatar = ctfAvatar.parent().children(".avatar.default");
  // if avatars are visible at all, create an in-memory
  // image element and set it's src to that of the ctf avatar. If
  // it loads successfully, toggle visibility of the two avatars.
  if (defaultAvatar.is(':visible')) {
    $("<img/>")
      .load(function() {
        ctfAvatar.show();
        defaultAvatar.remove();
      })
      .attr("src", ctfAvatar.attr("src"));
  }
}

function setSourceIcons() {
    $('img.source-icon').each(displaySourceIcon);
}

/**
 * displays the source icon if it can be successfully loaded from the external system (currently is only used for
 * work item source, and the icon is the icon for the source's tracker)
 */

function displaySourceIcon() {
    var source_icon = $(this);
    $("<img/>").load(function() {
                source_icon.parent().show();
            })
            .attr("src", source_icon.attr("src"));
}


(function() {
  //Set jQuery defaults
  jQuery.fx.interval = 20;

  var eventHandlers = new EventHandlers();

  // Add AJAX event handlers to the pages.
  eventHandlers.addAjaxErrorListener();

  // Set the target attribute for items with class external-link.
  // When the current window is an iframe, the links should open in top level window,
  // otherwise in a new tab.
  var linkTarget = (top == self) ? '_blank' : '_top';
  $('.external-link, a.external').attr('target', linkTarget);

  setAvatars();
  setSourceIcons();
})();




