/*
 * Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
 */

function AssociationTrace() {

    var me = this;
    var colorSet = {};
    var traceGraph;

    /**
     * displays the graph
     * @param data json association data
     */
    this.displayGraph = function (data, non_default_icon_url_map) {

        var canvasSize = me.getNewCanvasSize(window);

        // create the object with settings
        traceGraph = new $jit.RGraph({
            injectInto: 'trace_graph', //should be input to method as parameter
            transition: $jit.Trans.Expo.easeInOut,
            width: canvasSize.width,
            height: canvasSize.height,
            levelDistance: 125, //radius of the placement of nodes around the circle

            background: {
                levelDistance: 125, //radius of the background circles
                CanvasStyles: {
                    strokeStyle: '#999'
                }
            },

            Node: {
                overridable: true,
                color: '#000', //default color
                dim: 20 //radius of the canvas node
            },

            Navigation: {
                enable: true,
                panning: true,
                zooming: 100
            },

            onCreateLabel: function (elem, node) {
                var html = $('#node-label').clone();
                var nodeType = node['data']['class'];
                var icon_uri;

                if (nodeType == 'custom') {
                    if (node.id != undefined) {
                        icon_uri = non_default_icon_url_map[node.id];
                    }
                    var img = document.createElement('img');
                    img.src = icon_uri;
                    html.find('.node-type').addClass('work-item-node').append(img);
                    //if (node.id != undefined) {
                    //      icon_uri = $("#activity-widget-" + node.id).find(".source-type").css('background-image');
                    //    }
                    //
                    //html.find('.node-type').addClass('icons-' + nodeType);
                    //html.find('.icons-custom').css('background', icon_uri);
                } else if (nodeType == 'work-item') {
                    if (node.id != undefined) {
                        icon_uri = non_default_icon_url_map[node.id];
                    }
                    var img = document.createElement('img');
                    img.src = icon_uri;
                    html.find('.node-type').addClass('work-item-node').append(img);
                } else {
                    html.find('.node-type').addClass('icons-' + node['data']['class']);
                }

                html.find('.node-status').addClass(node['data']['status']);

                $(elem).html(html.html());

                if (node['data']['settled']) {
                    elem.style.cursor = 'pointer';
                }

            },

            //Place label centered on node
            onPlaceLabel: function (elem, node) {
                var style = elem.style;
                var left = parseInt(style.left);
                var top = parseInt(style.top);
                var width = elem.offsetWidth;
                var height = elem.offsetHeight;
                style.left = (left - width / 2) + 'px';
                style.top = (top - height / 2) + 'px';
            },

            Tips: {
                enable: true,
                type: 'Native',
                offsetX: 10,
                offsetY: 10,
                onShow: function (tip, node) {
                    var tip = $(tip);
                    tip.addClass('tip');
                    tip.html(me.getActivityWidget(node.id).html());
                }
            }

        });

        me.setupEvents(traceGraph);

        // load the graph data
        traceGraph.loadJSON(data);

        //color each node according to the css background color for each type of node
        //need to color each node by the source of the node if it is a custom
        traceGraph.graph.eachNode(me.assignColorForNode);


        // describe the method of animation for the graph
        traceGraph.fx.animate({
            modes: ['polar'],
            duration: 2000,
            hideLabels: false
        });

        // start the graph
        traceGraph.compute('end');
    };

    /**
     * Setup events
     * @param traceGraph the graph
     */
    this.setupEvents = function (traceGraph) {

        $(document).on('click', '.node', function (event) {
            me.doClick(window, event, me, traceGraph);
        });

        $(window).on('resize', function () {
            me.doResize(window, traceGraph);
        });

    };

    /**
     * Handles click events on the node labels
     * @param window the window
     * @param event the mouse click event
     * @param trace the association trace
     * @param traceGraph the graph object
     */
    this.doClick = function (window, event, trace, traceGraph) {
        var element = event.target;
        var nodeId = $(element).closest('.node').attr('id');

        var node = traceGraph.graph.getNode(nodeId);
        var settled = node['data']['settled'];

        if (nodeId) {
            if (event.metaKey || event.ctrlKey) {
                trace.replotGraph(window, nodeId);
            } else if (settled) {
                trace.openDetailsPageNewTab(window, nodeId);
            }
        }

        event.preventDefault();
    };

    /**
     * Calculates the new size that we need for the canvas
     * @param windowSelector the window
     * @returns {{height: number, width: number}} a hash with the height and width of the expected window.
     */
    this.getNewCanvasSize = function (windowSelector) {
        var header = $('#header'),
            filter = $('.activity-filter'),
            jWindow = $(windowSelector),
            ieCorrection = 1, //IE has an off-by-one error on height calculations
            height,
            width;

        height = jWindow.height() - header.height() - filter.height() - ieCorrection;
        width = jWindow.width();
        return {height: height, width: width};
    };

    /**
     * Resizes the association trace graph to fill the available space in the page
     * @param windowSelector the window selector (for testing really)
     * @param traceGraph the jit.RGraph to resize
     */
    this.doResize = function (windowSelector, traceGraph) {
        var size = this.getNewCanvasSize(windowSelector);
        traceGraph.canvas.resize(size.width, size.height);
    };

    /**
     * Creates a new search string using the given item ID, preserves other existing params.
     * @param item_id the item ID for the new query
     * @returns {string} the new search string
     */
    this.getSearchString = function (item_id) {
        return me.currentSearchString().replace(/%5Bquery%5D=\w+/, '%5Bquery%5D=' + item_id)
    };


    /**
     * Gets the current search string
     * @returns {string} the current search string
     */
    this.currentSearchString = function () {
        return window.location.search.substring(1);
    };

    /**
     * Assigns the $color of a jit Graph.Node based on the css class in the node data.
     * @param node the jit Graph.Node to add the color to
     */
    this.assignColorForNode = function (node) {
        var cssClass = node.data['class'];
        if (cssClass == "custom") {
            var color = me.findColorInCss(cssClass, node.id);
        }
        else {
            var color = me.findColorInCss(cssClass);
        }

        node.data.$color = color;
    };

    /**
     * Finds the color of the activity type in CSS.
     * @param {node} node will provide the cssClass and node.id to set the color based on the node.id's source background color
     * @returns {String} color the color of the activity type returned as "rgb(x, y, z)"
     */
    this.findColorInCss = function (cssClass, nodeId) {
        var color = colorSet[cssClass];

        if (nodeId != undefined) {
            color = $("#activity-widget-" + nodeId).children(".custom")
                .children(".activity-contents").children(".activity-icons").css('background-color');
        }//end if the node represents a custom activity, dig down into the widget to get the background-color

        if (color == null) {
            color = $('.' + cssClass).children('.activity-icons').css('backgroundColor');
            colorSet[cssClass] = color;
        }

        return color;
    };

    /**
     * Returns the set of found colors stored in the object
     * @returns {Object} hash of activity types and the color associated to each activity type
     */
    this.getColorSet = function () {
        return colorSet;
    };

    /**
     * Gets the activity widget html as a jQuery object.
     * @param nodeId the opaque id of the node
     * @returns {*|jQuery|HTMLElement}
     */
    this.getActivityWidget = function (nodeId) {
        return $('#activity-widget-' + nodeId);
    };

    /**
     * Gets the href link for the details page from the activity widget.
     * @param nodeId the opaque id of the node
     * @returns {*}
     */
    this.getActivityDetailPath = function (nodeId) {
        return me.getActivityWidget(nodeId).find('a').attr('href');
    };

    /**
     * Opens the detail page for the given activity for the given opaque id.
     * @param window the window
     * @param nodeId the opaque id of the node
     */
    this.openDetailsPageNewTab = function (window, nodeId) {
        window.open(me.getActivityDetailPath(nodeId), "_blank");
    };

    /**
     * Re-plots the graph using the given node id
     * @param window the window
     * @param nodeId the node id
     */
    this.replotGraph = function (window, nodeId) {
        window.location.search = me.getSearchString(nodeId);
    };

}
