/*
 * Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
 */

/**
 * Handles updating page elements based on user interaction with comments.
 */
function Comments() {
  var me = this;

  var commentTextArea = $('.comment-textarea');
  var originalHeight = commentTextArea.outerHeight();
  var originalScrollHeight = commentTextArea.prop('scrollHeight');

  var commentSubmit = $('.comment-submit');

  /**
   * Updates the button state.
   */
  this.updateButtonState = function() {
    var toggle =  !$.trim(commentTextArea.val()) || commentSubmit.hasClass('in-flight');
    commentSubmit.toggleClass('disabled', toggle);
  };

  /**
   * Expands the text area
   */
  this.updateTextAreaHeight = function() {
    if(this.rows == 1) {
      this.style.height = originalScrollHeight * 4 + 'px';
      this.rows = 4;
    }
  };

  /**
   * Performs needed page updates when submit fails.
   * @param xhr unused
   * @param data the data, uses responseText property
   */
  this.onAjaxError = function(xhr, data) {
    commentSubmit.removeClass('in-flight');
    commentTextArea.addClass('error');

    $('.comment-error').html(data.responseText).removeClass('hidden');

    me.updateButtonState();
  };

  /**
   * Performs needed page updates when submit succeeds
   * @param xhr unused
   * @param data the successful comment
   */
  this.onAjaxSuccess = function(xhr, data) {
      $(data).hide().prependTo('.user-comments').fadeIn('slow');

      // update the selected association's counter
      me.incrementCommentCount($('.selected .activity-comment-count'));
      setAvatars();

      commentTextArea.removeClass('error').val('').height(originalHeight).attr('rows', '1');
      commentSubmit.removeClass('in-flight');

      $('.comment-error').addClass('hidden');
  };

    /**
     * Extracts a count integer from a DOM element and increments it.  Removes 'hidden'
     * class from the element if present.  Counts beyond 99 display "99+"
     * @param commentCounter The comment counter DOM element.
     */
  this.incrementCommentCount = function(commentCounter) {
      var currentCounterTotal = commentCounter.html();
      if (currentCounterTotal.indexOf("99+") == -1) {
          var new_count = parseInt(currentCounterTotal) + 1;
          if(new_count > 99) {
              new_count = "99+";
          }
          commentCounter.html(new_count);
      }
      commentCounter.removeClass('hidden');
  }

  /**
   * Sets up the form events.
   */
  this.setFormEvents = function(){

    $('body').die('ajax:error'); // we'll handle our own errors

    $('.comment-form')
      .on('ajax:error', this.onAjaxError)
      .on('ajax:success', this.onAjaxSuccess);

    // submits form when button is clicked
    commentSubmit.on('click', function() {
      $(this).not('.disabled').addClass('disabled in-flight').parent('form').trigger('submit.rails');
    });

    // Resizes the text area to 4x what it was (aka 4 lines) on focus
    commentTextArea
      .on('focus', this.updateTextAreaHeight)
      .on('keyup change mouseup paste cut', this.updateButtonState);
  };
}

(function (){
  var comments = new Comments('.comment-form');
  comments.setFormEvents();
  pageDone(true);
})();
