/*
 * Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
 */

/**
 * Manages dropdown functionality.
 */
var SourceTypeDropDown = (function() {
  var SourceTypeDropDown = function() {
    this.dropDownSelector = '.dropdown-selector';
    this.currentValueSelector = '.current.dropdown-value';
    this.dropDownValueSelector = '.dropdown-value';
    this.dropDownArrowSelector = '.dropdown-arrow';
  };
  
  SourceTypeDropDown.prototype = {
    /**
     * Binds events needed for combo box functionality.
     */
    initialize: function() {
      $(this.dropDownSelector).on('click', $.proxy(this.toggleDropDown, this));
      $(document).on('click', $.proxy(this.onClickAway, this));
    },

    /**
     * Closes the dropdown if the event target is not a part of the dropdown.
     * @param event the jquery event
     */
    onClickAway: function(event) {
      var target = $(event.target);
  
      if (target.parents(this.dropDownSelector).length == 0) {
        var originalSelection = $(this.currentValueSelector);
        this.toggleClassForCurrentValue(originalSelection, false);
        $(this.dropDownSelector).html(originalSelection);
      }
    },

    /**
     * Opens and closes the dropdown, selecting the chosen element if an element was chosen
     * @param event the jquery event
     */
    toggleDropDown: function(event) {
      var target = ($(event.target).parents(this.currentValueSelector).length === 1) ?
          $(this.currentValueSelector) :
          $(event.target)
  
      if($(this.dropDownSelector).children().length == 1) {
        this.toggleClassForCurrentValue(target, true);
        this.expandDropDown();
      } else {
        this.selectElement(this.findSelectedValue(target));
      }
    },
  
    /**
     * Expands the dropdown
     */
    expandDropDown: function() {
      var clone = $('.dropdown-values').clone().removeClass('hidden');
      //do not display the default ("Select ...") twice when the default option is present in the dropdown. Note that
      // this 'option' is expected not to have the data attribute unlike other options
      if($(this.currentValueSelector).data("dropdown-widget-option-value") === "") {
        clone.find('.option[data-dropdown-widget-option-value=""]').remove();
      }
      $(this.dropDownSelector).append(clone);
    },
  
    /**
     * Changes a css class on an option (current dropdown value) to give a dropdown a more native look when it is open.
     * Also replaces the closed-arrow with open-arrow, and vice-versa.
     * @param {jQuery} option the dropdown element that needs to be modified.
     * @param {Boolean} open true if setting the icon to be open, false if closed
     */
    toggleClassForCurrentValue: function(option, open) {
  
      var openIcon = 'icons-accordion-open-arrow';
      var closedIcon = 'icons-accordion-closed-arrow';
  
      var remove, add;
  
      if(open) {
        remove = closedIcon;
        add = openIcon;
        option.addClass('open');
      } else {
        remove = openIcon;
        add = closedIcon;
        option.removeClass('open');
      }
  
      option.find(this.dropDownArrowSelector)
          .removeClass(remove)
          .addClass(add);
    },
  
    /**
     * Finds the selected dropdown value based on the target element. Or returns the currently selected value if no others
     * were selected. Used to determine if a new selection has been made, or to close the dropdown with out selecting an
     * element.
     *
     * @param {jQuery} target the jqueried target element
     * @return {jQuery} the jQuery object of the selected dropdown value.
     */
    findSelectedValue: function(target) {
      var selection = target.parents(this.dropDownValueSelector);
  
      if(target.is(this.dropDownValueSelector)) {
        selection = target;
      } else if(selection.length == 0) {
        selection = $(this.currentValueSelector);
      }
  
      return selection;
    },
  
    /**
     * Clears the dropdown and replaces the old selected element with the newly selected element
     * @param selection the newly selected element
     */
    selectElement: function(selection) {
      var hidden_input_field, current_selection, option_value;

      selection = this.makeCurrentValue(selection);
  
      $(this.dropDownSelector).html(selection);
      hidden_input_field = $('.form-dropdown-widget-hidden-value');
      current_selection = $(this.currentValueSelector);
      option_value = 'dropdown-widget-option-value';

      //swap the forms if the value of the commit_server type has changed
      if (current_selection.data(option_value) !== hidden_input_field.val()) {
        hidden_input_field.val(current_selection.data(option_value)).change();
      }
    },
  
    /**
     * Modifies classes to make the element the current selected value.
     * @param element the selected element
     * @return element modified to be a current selected element
     */
    makeCurrentValue: function(element) {
      element.find(this.dropDownArrowSelector).removeClass('hidden');
  
      element.addClass('current');
  
      this.toggleClassForCurrentValue(element, false);
  
      return element;
    }
  };
  
  return SourceTypeDropDown;
})();
