#
# Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

# Handles activity view requests
require 'will_paginate'

class ActivityController < ProjectScopedController

  # We need to reset the session to prevent using exiting session
  # when a user directly goes page which does not trigger TeamForge session check if there is session exist.
  before_action :reset_session

  layout 'activity_details', only: [:show]

  # Handles /activities requests. Responds with an HTML response.
  def index
    @filter = ActivityFilter::Activities.new(params[:filter])
    @filter.activity_types = current_project.activity_types
    @max_atomic = Activity.order(:atomic).last.try(:atomic)
    @activities = @filter.activities current_project, page_number: params[:page].to_i,
                                                       page_item_count: Orchestrate::Application.config.activities_per_page

    # use to cache all the object and trigger identity map later on
    @activity_summaries = ActivitySummary.where(:id => @activities.map(&:activity_summary_id)).to_a

    flash.now[:error] = @filter.error_messages unless @filter.valid?
  end

  # Shows an individual activity, located via its opaque ID
  # If it's not found, a 404 error is thrown.
  def show
     @activity = current_project.find_activity_summary_by_opaque_id(params[:id])
     @activities = @activity.all_activity_summary_associations(true)

    # TODO(mschweers) - Fix this monstrosity by adding some self-awareness to ActivitySummary descendants.
    render "#{(@activity.class.to_s).gsub('Summary', '').pluralize.underscore}/show"
  end

  # Renders json with the count of all activities (unfiltered!) since the activity_id (opaque id) provided.
  # TODO - support the filters.
  def count
    atomic_id = params[:activity_id].to_i

    @count = current_project.count_newer(atomic_id)
  end
end
