#
# Copyright 2015 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#
# Permissions for Schemas Controller:
#   :create == :site_admin || :pipeline_create
#   :index && :show == only need Teamforge Authentication to view
#
# Controller for viewing and creation of Schemas via HTTP API

class Api::V1::SchemasController < Api::BaseController
  #Authenticates API calls with TeamForge
  include CtfApiAuthentication
  include Api::SchemasHelper


  before_action :authenticate_with_teamforge_and_get_permissions
  before_action :check_post_schema_permissions_for_user, only: [:create]

  # Get a list of all the schemas
  def index
    render json: CustomSchema.order(:schema_id.asc, :schema_version.asc).all
  end

  # Get the schema with the provided schema_id & optionally schema_version
  def show
    if params[:schema_id].present?
      unescaped_schema_id = CGI::unescape(params[:schema_id])
      schema = if params[:schema_version].present?
                 CustomSchema.where(:schema_id => unescaped_schema_id, :schema_version => params[:schema_version].to_i)
               else
                 CustomSchema.where(:schema_id => unescaped_schema_id)
               end

      if schema.present?
        render json: schema
      else
        head status: 404
      end
    end
  end


  def create
    begin
      schema = request.raw_post
      send_schema_to_queue(schema)
    rescue Exception => ex
      logger.error(ex.message)
      render html: ex, status: 404
    else
      schema_url = create_url_for_schema(schema)
      render html: schema_url
    end
  end

end
