#
# Copyright 2015 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

class Api::V1::ToolsController < Api::BaseController

  #Authenticates API calls with TeamForge
  include CtfApiAuthentication

  before_action :authenticate_with_teamforge_and_get_permissions
  before_action :can_crud_tools?
  before_action :get_project

  def index
    tools = Tool.by_project(current_project.id).all
    render json: { tools: tools }, status: :ok
  end

  def show
    tool = Tool.where(id: params[:id]).by_project(current_project.id).first
    if tool
      render json: { tool: tool }, status: :ok
    else
      head :not_found
    end
  end

  def create
    tool = Tool.new(params[:tool].merge(project: current_project))
    if tool.save && tool.errors.empty?
      render json: { tool: tool}, status: :ok
    else
      render json: { errors: tool.errors.messages }, status: :unprocessable_entity
    end
  end

  def update
    generic_tool_updater(params[:tool])
  end

  def activate
    generic_tool_updater({active: true})
  end

  def deactivate
    generic_tool_updater({active: false})
  end

  def is_name_unique
    render json: { is_name_unique: Tool.name_unique?(current_project, params[:name]) }, status: :ok
  end

  def generic_tool_updater(values)
    tool = Tool.where(id: params[:id]).by_project(current_project.id).first
    if tool 
      if tool.update_attributes(values)
        head :ok
      else
        render json: { errors: tool.errors.messages }, status: :unprocessable_entity
      end
    else
      head :not_found
    end
  end
  private :generic_tool_updater

end
