#######################################################################################
# REST interface point to interact with the Emulator Service from the CTF interface
# Entire GET, POST, PUT  set of operations is supported
# DELETE is not supported. The user cannot delete the emulation table
# Signature for the Base URI :
#    GET|POST|  /api/3/projects/:project/webhook/emulation/
#
# copyright collabnet, 2015
# ######################################################################################

class Api::V5::Webhook::EmulationController < Api::BaseController

  #######################################################################
  # GET /api/:version/projects/:project_id/webhook/emulation
  #######################################################################

  def index

    # get the emulation table
    ret = Emulator.interface('', '', 'GET')

    if ret == [] || ret.present?
      response = {'emulation_table' => ret, status: '200'}
    else
      response = {error: 'bad request', status:  '400'}
    end

    render :json => response
  end


  #######################################################################
  # GET /api/:version/projects/:project_id/webhook/emulation/:id
  #######################################################################

  def show

    # get the emulation table
    ret = Emulator.interface('', params[:id], 'GET')

    if ret == [] || ret.present?
      response = {'emulation_table' => ret, status: '200'}
    else
      response = {error: 'bad request', status:  '400'}
    end

    render :json => response
  end



  #######################################################################
  # POST /api/:version/projects/:project_id/webhook/emulation
  # 400 = bad HTTP request
  #######################################################################

  def create

    #-------------------------------------------------------
    # request emulator service to persist an emulation table
    #-------------------------------------------------------

    begin
      emulator = params['emulation']
      ret = Emulator.interface(emulator, '', 'POST')

      if ret
        response = { emulator: ret, status: "200" }
      else
        response = { error: 'bad request', status: "400" }
      end
    rescue => e
      response = { error: 'bad emulation table', status: "400" }
    end

    render :json => response
  end



  ################################################################
  # DELETE /api/3/projects/:project_id/webhook/activity_maps/:id
  ################################################################

  def destroy

    #--------------------------------------------------------
    # destroy an activity map in the activity maps collection
    #--------------------------------------------------------

    ret = Emulator.interface(nil, params['id'], 'DELETE')

    if ret
      response = { response: params['id'], status: '200' }
    else
      response = { error: 'emulator table not deleted', status: '404' }
    end

    render :json => response
  end


  ################################################################
  # PUT -  /api/3/projects/:project_id/webhook/activity_maps/:id
  ################################################################

  def update

    #--------------------------------------------------------
    # update an activity map in the activity maps collection
    #--------------------------------------------------------

    ret = Emulator.interface(params['emulation'], params['id'], 'PUT')

    if ret
      response = { response: params['id'], status: '200' }
    else
      response = { error: 'emulator table not updated', status: '404' }
    end

    render :json => response
  end

end