#######################################################################################
# REST interface point to interact with the PacketMap Service from the CTF interface
# Entire CRUD set of operations is supported
# Signature for the URI :
#    GET|POST|PUT|DELETE  /api/3/projects/:project/webhook/packet_maps/[:id]
#
# copyright collabnet, 2016
# ######################################################################################


class Api::V5::Webhook::PacketMapsController < Api::BaseController

  #######################################################################
  # GET /api/:version/projects/:project_id/webhook/packet_maps
  #######################################################################

  def index

    # get the packet maps
    response = PacketMaps.interface('get')

    if response.blank?
      response = {error: 'bad request', status: '400'}
    else
      response = {'packet_map' => response, status: '200'}
    end

    render :json => response
  end


  #################################################################################
  # GET /api/:version/projects/:project_id/webhook/packet_maps/:packet_type/
  #################################################################################

  def show

    packet_type = params['packet_type']

    # get a packet map
    response = PacketMaps.interface('get', nil, nil, packet_type )

    if response.blank?
      response = {error: 'bad request', status: '400'}
    else
      response = {'packet_map' => response, status: '200'}
    end

    render :json => response
  end


  #######################################################################
  # POST /api/:version/projects/:project_id/webhook/packet_maps/
  # 400 = bad HTTP request
  #######################################################################

  def create

    #----------------------------------------------------------
    # request the ActivityMap service to persist a packet map
    #----------------------------------------------------------

      packet_map = params['packet_map']

      ret = PacketMaps.interface('post', packet_map, nil, nil)

      if ret
        response = { 'packet_map' => ret, status: "200" }
      else
        response = { error: 'bad request', status: "400" }
      end

    render :json => response
  end


  ###############################################################
  # PUT /api/3/projects/:project_id/webhook/packet_maps/:id
  # search key for subject pattern is params['group_id']
  ##############################################################

  def update

    #-------------------------------------------------------
    # update a packet map in the packet map collection
    #-------------------------------------------------------

    ret = PacketMaps.interface('put', params['packet_map'], params['group_id'], nil)

    if ret
      response = { packet_map: ret, status: '200'}
    else
      response = { error: 'failed to update document', status: '404' }
    end

    render :json => response
  end

  ####################################################################
  # DELETE /api/3/projects/:project_id/webhook/packet_maps/:group_id
  ####################################################################

  def destroy

    #--------------------------------------------------------
    # destroy an packet map in the packet maps collection
    #--------------------------------------------------------

    ret = PacketMaps.interface('delete', nil, params['group_id'], nil)

    if ret
      response = { response: params['group_id'], status: '200' }
    else
      response = { error: 'document not deleted', status: '404' }
    end

    render :json => response
  end

end
