#######################################################################################
# REST interface point to receive data packets from external sources
# The only permitted action on an incoming resource is create.
#
# Packets URL:
#  Path to controller and method packets :
#
#         POST /api/3/projects/:project_id/webhook/webhooks/:packet_type/:sak/
#
# create is the only REST action allowed for packets
#
# the packet type must belong to a predefined set: { 'github', 'jira'   }
# The id_string variable will be populated by JIRA. This will be
# primary key pertaining to the packet (called remote_id in Orchestrate). For JIRA it
# will have the format ${issue.key} for Jira
# copyright collabnet, 2015
#########################################################################################

 class Api::V5::Webhook::PacketsController < Api::BaseController


   ##########################################################################
   # POST /api/3/projects/:project_id/webhooks/packets/:packet_type/:sak/
   # invoke the datadump create to create packet resources for the
   # DataDump Collection
   # return a status code error if the create operation fails
   # or return the created object and a success HTTP status
   # status code 400 : bad request
   ##########################################################################

  def create

     #-------------------------------------------------------
     # persist into datadump, any views and the legacy system
     #-------------------------------------------------------
     if params['packet'].present?

       ret = DataDump.interface(params['packet'], request.url)

       if ret.blank?
         response = { error: 'cannot persist data packet', status: '400' }
       elsif ret == true
         response = { status: '200' }
       elsif ret.is_a?(Array)
         packet = {packet: ret[2]}
         response = packet.merge({status:  "200", 'views inserted into' => ret[0], 'packets_queued' => ret[1] })
       end

     else
       response = { error: 'packet does not have an identified packet type', status:  "400" }
     end

     render :json => response
  end

end
