#########################################################################################
# REST interface point to interact with patterns
# Entire CRUD set of operations is supported

# Signature for the Base URI :
#    GET|PUT|POST|DELETE  /api/3/projects/:project/webhook/patterns/[:id]
#
# copyright collabnet, 2015
# ######################################################################################

class Api::V5::Webhook::PatternsController < Api::BaseController

  ###################################################################
  # GET /api/:version/projects/:project_id/webhook/patterns
  ###################################################################

  def index

    # get all patterns
    ret = Pattern.interface('',  'GET')

    if ret.blank?
      response = {error: 'bad request', 'status' => '400'}
    else
      response = {'patterns' => ret, 'status' => '200'}
    end

    render :json => response
  end


  #######################################################################
  # GET /api/:version/projects/:project_id/webhook/patterns/:pattern_id
  #######################################################################

  def show

    #----------------------------------------------------------
    # return a specific  pattern from the patterns collection
    #----------------------------------------------------------

    ret = Pattern.interface(nil, 'GET', params['pattern_id'])

    # http status code 200
    if ret
      response = { pattern: ret, status: '200'}
    else
      response = { error: 'bad request', status: '400'}
    end

    render :json => response

  end


  #############################################################
  # POST /api/3/projects/:project_id/webhook/patterns/
  #############################################################

  def create

    #-----------------------------------------------------
    # ask pattern service to persist the received pattern
    # into the patterns collection
    #-----------------------------------------------------

    ret = Pattern.interface(params['pattern'], 'POST')

    if ret
      response = params['pattern'].to_hash.merge({ 'status' => '200' })
    else
      response = { error: 'bad request', status: '400' }
    end

    render :json => response

  end


  ##############################################################
  # PUT /api/3/projects/:project_id/webhook/pattern_id/:pattern
  # search key for subject pattern: params['pattern_id']
  ##############################################################

  def update

    #----------------------------------------------
    # update a pattern in the patterns collection
    #----------------------------------------------
    ret = Pattern.interface(params['pattern'], 'PUT', params['pattern_id'])

    # pattern updated
    if ret
      response = { pattern_id: params['pattern_id'], status: '200'}
    else
      response = { error: 'bad request', status: '400' }
    end

    render :json => response
  end


  ################################################################
  # DELETE /api/3/projects/:project_id/webhook/patterns/:id
  ################################################################

  def destroy

    #----------------------------------------------
    # destroy a pattern in the patterns collection
    #----------------------------------------------

    ret = Pattern.interface(nil, 'DELETE', params['pattern_id'])

    if ret
      response = { pttern_id: params['pattern_id'], status: '200' }
    else
      response = { error: 'bad request', status: '404' }
    end

    render :json => response
  end

end
