#
# Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

class AssociationTraceController < ProjectScopedController

  include ApplicationHelper
  include AssociationTraceHelper

  # Returns data in the format expected by the graph
  def index
    @filter = ActivityFilter::AssociationTrace.new(params[:filter])
    @activities = {}
    
    @activity = Node.find_activity_by_id(@filter.query)

    if @activity
      @data = @filter.node(current_project, @activity, @activities).generate_graph_model
    end

    @filter_header_data = {
        activity_count: @data ? calculate_activity_count(@data) : 0,
        activity: @activity.blank? ? @filter.query : activity_display_name(@activity, true)
    }

    @non_default_icon_map = {}
    process_data_to_map_work_item_icon_urls([@data]) unless @data.blank?

    flash.now[:error] = @filter.error_messages unless @filter.valid?
  end

  # breadcrumbs navigation for association_trace_controller
  # @return [Hash] of display text/href for the breadcrumbs
  def breadcrumbs
    breadcrumbs_activities.tap do |breadcrumbs_hash|
      breadcrumbs_hash[t("menubar.trace")] = trace_path(filter: { query: @activity })
    end
  end

  private
  def process_data_to_map_work_item_icon_urls(nodes = [])

    nodes.each do |node|
      if node[:data][:class] == 'work-item'
        source_for_the_activity = WorkItemSummary.where(opaque_id: node[:id]).first.source_id
        work_item_icon_url = Tracker.where(work_item_source_id: source_for_the_activity).first.icon
        @non_default_icon_map[node[:id]] = work_item_icon_url
      elsif node[:data][:class] == 'custom'
        source_for_the_activity = CustomSummary.where(opaque_id: node[:id]).first.source_id

        # This xds_icon uri is used non-rails image display method. So, icon uri should contains proxy path.
        # ex: /orc/assets/icons/custom_source/image.png
        xds_icon = CustomActivitySource.normalize_orchestrate_icon_uri(CustomActivitySource.where(_id: source_for_the_activity).first.icon_uri, true)

        @non_default_icon_map[node[:id]] = xds_icon
      end

      if node[:children].size > 0
        process_data_to_map_work_item_icon_urls(node[:children])
      end
    end
  end

end
