#
# Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

# Controller for comments system
class CommentsController < ProjectScopedController

  attr_writer :clock

  # Create a comment in the database
  def create
    commentable = current_project.find_activity_summary_by_opaque_id(params[:id])

    content = params[:comment]
    content.strip! unless content.nil?

    comment = Comment.new(
        comment: content,
        time_created: clock.now,
        user: current_user,
        commentable: commentable)
    
    if comment.valid?
      existing_comments = commentable.comments
      commentable.comments = [comment].concat(existing_comments)
      commentable.save
      render partial: 'comments/comment', locals: {comment: comment, count: commentable.comments.count,
                                                    comment_counter: 0}
    else
      render partial: 'shared/field_error', locals: { target: comment, field: :comment}, status: :not_acceptable
    end
  end

  private

  # Gives the clock to use for this controller.
  # @return [Time] a Time class if a clock has not explicitly been set
  def clock
    @clock || Time
  end

end
