#
# Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

class CustomActivitySourcesController < ActivitySourceController
  before_filter :activity_sources, only: [:edit, :update, :create]
  before_filter :available_tags, only: [:new, :edit]

  def index
    sources_index(params)
  end

  def new
    @custom_activity_source = @source_server = CustomActivitySource.new
    @activity_sources = current_project.sources
    render template: 'shared/crud_form', locals: {source_server: @custom_activity_source, activity_sources: @activity_sources}
  end

  def create
    params[:custom_activity_source][:activity_source_tags] = extract_tags(params[:custom_activity_source])
    add_source_to_project(
        source_type: CustomActivitySource,
        source_attributes: params[:custom_activity_source],
        saved_message: t('custom_activity_source.messages.source_created'),
        not_created_message: t('custom_activity_source.messages.source_not_created'))
  end

  def edit
    render_crud_form(@custom_activity_source)
  end

  def update
    @custom_activity_source.display_name = params[:custom_activity_source][:display_name]
    @custom_activity_source.activity_source_tags = extract_tags(params[:custom_activity_source])
    icon_background_color = params[:custom_activity_source][:background_color_value]
    @custom_activity_source.background_color_value = icon_background_color if icon_background_color
    @custom_activity_source.icon_uri = params[:custom_activity_source][:icon_uri]

    ############################################################
    # update the associated activity source only if the source
    # already does not have an associated source
    ############################################################

    if !@custom_activity_source.activity_source_id
      @custom_activity_source.activity_source_id = params[:custom_activity_source][:activity_source_id]
    end

    #########################################################
    #  update the trusted sources array by adding new
    #  trusted sources to the trusted sources array
    #########################################################

    @custom_activity_source.trusted_sources ||=  []

    if params[:custom_activity_source][:trusted_sources].present?
      @custom_activity_source.trusted_sources += params[:custom_activity_source][:trusted_sources]
      @custom_activity_source.trusted_sources.uniq!
    end

    if @custom_activity_source.save
      flash[:success] = t('custom_activity_source.messages.source_updated')
      redirect_to custom_activity_sources_path(@custom_activity_source.project)
    else
      available_tags
      render_crud_form(@custom_activity_source)
    end
  end

  # Finds the work item source by its ID or raises an error if it's not found.
  def find_source
    @source_server = @custom_activity_source = CustomActivitySource.where(id: params[:id], project_id: current_project.id).first
    raise MongoMapper::DocumentNotFound.new(t("custom_activity_source.errors.not_found")) unless @custom_activity_source
  end

  def activity_sources
    @activity_sources = current_project.sources - [@custom_activity_source]
  end

  def active_status
    update_active_from_param(@custom_activity_source, params[:new_status])
    path_opts = path_options_with_project_context(action: :edit)
    redirect_to polymorphic_path(@custom_activity_source, path_opts)
  end

  def canonicalize_server_params
    @server_params = params[:custom_activity_source]
  end

  def extract_tags(param_hash)
    param_hash.delete(:activity_source_tags).to_s.split(',').map do |tag_name|
      ActivitySourceTag.first_or_create(name: tag_name.downcase)
    end
  end

end
