#
# Copyright 2014 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

# Controller for the Projects overview page.

class ProjectController < ProjectScopedController
  # TODO: this filter should be removed altogether in favor of allowing application_controller
  # to handle this sort of logic.

  # Skip extract ctf session for sources_overview. Sources overview is
  # accessed after ctf session info is extracted.
  before_filter :require_project, only: [:show]
  before_filter :authorize_for_edit_create_permissions!, only: [:sources_overview]
  skip_before_filter :authorize_for_project!
  helper_method :breadcrumbs

  def index
    tf_project = extract_ctf_session
    if tf_project && tf_project.project
      redirect_to activity_index_path(tf_project.project_url)
    else
      # There is no project context, redirect to ctf all projects page
      redirect_to CtfGoLinks::all_projects_url
    end
  end

  def show
    if params[:project].nil?
      redirect_to :action => :index
    else
      redirect_to activity_index_path(params[:project])
    end
  end

  def sources_overview
    @source_types = ActivitySource.source_types
    @selected_source = @source_types.keys[0]
    source_type = @source_types.values[0]
    @project_selected_sources = current_project.sources.types(source_type)
  end


  protected
  def breadcrumbs
    Hash.new.tap do |breadcrumbs_hash|
      breadcrumbs_hash[current_project.name] = current_project ? root_path(project: current_project) : root_path
      breadcrumbs_hash[t("menubar.sources_overview")] = sources_overview_path(current_project) if current_project && params[:action] == 'sources_overview'
    end
  end

  def require_project
    not_found ( t('errors.project.not_found', name: "#{params[:project]}") ) if current_project.nil?
  end

  # Extracts CTF project information from the URL and loads the project data from our database. We set session
  # variables to remember that we're using CTF.
  def extract_ctf_session
    has_context = has_project_context?(params, session)
    tf_project = nil
    if has_context
      # This is where we would log into CTF using the sfLoginToken to get a more permanent soap session ID. That
      # soap session ID could go into the session to make requests to CTF as the current user.

      if params[:sfProj] || params[:project]
        proj_url = params[:sfProj] || params[:project]
        tf_project = TeamForge::ProjectConfiguration.find_by_project_url(proj_url)
      else
        # Either sfproj or sfProjId should be available if it reached this far.
        tf_project = TeamForge::ProjectConfiguration.find_by_ctf_project_id(session[:sfProjectId])
      end
    end

    unless tf_project.nil?
      session[:ctf_project] = cookie_for_ctf(tf_project)
      # TODO: For parity with ApplicationController#extract_project_context, remove url from session if loaded
      # via parameter context
      session[:ctf_project].try(:delete, :project_url)
    end
    tf_project
  end

  # Creates a hash to store in a session cookie.
  # @param [TeamForge::ProjectConfiguration] ctf_configuration configuration for the project used to create the hash
  # @return [Hash] the data store in the cookie
  def cookie_for_ctf(ctf_configuration)
    return nil unless ctf_configuration
    {
        server_url: ctf_configuration.server_url,
        project_url: ctf_configuration.project_url,
        link_plug_id: ctf_configuration.link_plug_id
    }
  end

  def has_project_context?(params, session)
    (params[:sfProj] && params[:sfLoginToken] && params[:isLoggedIn]) || (session[:sfProjectId].present?)|| params[:project].present?
  end
end
