#
# Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

# Controller for the code review servers.
class ReviewServersController < ActivitySourceController

  def index
    sources_index(params)
  end

  # Shows the form for creating a new build server.
  def new
    @review_server = ReviewServer.new
    render template: 'shared/crud_form', locals: {source_server: @review_server}
  end

  # Edits a review server.
  def edit
    render_crud_form(@review_server)
  end

  # Updates a review server.
  def update
    if @review_server.update_attributes(params[:review_server])
      flash[:success] = t("review_server.messages.server_updated")
      redirect_to review_servers_path(@review_server.project)
    else
      render_crud_form(@review_server)
    end
  end

  # Creates a review server.
  def create
    add_source_to_project(
        source_type: ReviewServer,
        source_attributes: params[:review_server],
        saved_message: t('review_server.messages.server_created'),
        not_created_message: t('review_server.messages.server_not_created'))
  end

  # Sets the active status on a review server. The params[:new_status] must be one of "active" or "inactive". Anything
  # else returns does nothing.
  def active_status
    update_active_from_param(@review_server, params[:new_status])
    path_opts = path_options_with_project_context(action: :edit)
    redirect_to polymorphic_path(@review_server, path_opts)
  end

  def canonicalize_server_params
    @server_params = params[:review_server]
  end

  private
  # Loads the particular review server given by the params[:id].
  # @raise [MongoMapper::DocumentNotFound] if the review server was not found
  def find_source
    @source_server = @review_server = ReviewServer.where(id: params[:id], project_id: current_project.id).first
    raise MongoMapper::DocumentNotFound.new(t("review_server.messages.not_found")) unless @review_server
  end

end
