#
# Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

module TeamforgeApplication
 include SharedTeamforgeApplication

  def self.included(base)
    base.class_eval do
      before_filter :ping_teamforge
      before_action :refresh_timezone_for_user
      before_filter :ctf_permissions_check
    end
  end

  # Pings the TeamForge server to make sure we're still logged in and to refresh our soap session.
  def ping_teamforge
    @ctf_session = CtfService.create_teamforge_session(session['ctf_soap_token'])
    tf = @ctf_session.teamforge
    @ctf_user = tf.get_current_user_data(session_id: tf.session_id)
    @ctf_user = @ctf_user[:multi_ref] || @ctf_user
    render_unauthorized if %w(Disabled Removed Pending).include? @ctf_user[:status]
  rescue TeamForge::ServiceError => e
    # thrown when the user is logged out of CTF
    Rails.logger.debug(e)
    remember_path #need this for the proper redirect
    logout_user
  end

  def logout_user
    sign_out current_user
    session['ctf_soap_token'] = nil
    redirect_to new_user_session_path
  end

  add_method_tracer :ping_teamforge
  add_method_tracer :ctf_permissions_check
end
