#
# Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

class Users::OmniauthCallbacksController < ActionController::Base
  # Endpoint for authentication callbacks from teamforge strategy
  def teamforge
    authenticate
    session['ctf_soap_token'] = request.env['omniauth.auth'].credentials[:token]
    # TODO: Remove this when SWL Orchestrate goes away
    session['sfProjectId'] = params['sfProjectId']
  end

  # Endpoint for authentication callbacks from localauth strategy
  def localauth
    user = authenticate
    user.refresh_permissions request.env['omniauth.auth'].info
  end

  # Endpoint for authentication failures. Just renders our 401 page.
  def failure
    render file: "public/401", formats: [:html], status: :unauthorized
  end

  private

  # Shared endpoint for authentication callbacks from omniauth
  def authenticate
    auth_hash = request.env['omniauth.auth']
    user = User.find_or_create_user(auth_hash)
    user.after_login(auth_hash)
    sign_in_and_redirect user, event: :authentication
    user
  rescue => e
    Rails.logger.debug(e.message)
    Rails.logger.debug(e.backtrace)
    raise
  end
end
