#
# Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

# Controller for the Work Item source add/edit page.

class WorkItemSourcesController < ActivitySourceController

  def index
    sources_index(params)
  end

  def new
    render template: 'shared/crud_form', locals: {source_server: WorkItemSource.new}
  end

  def edit
    render_crud_form(@work_item_source)
  end

  def create
    add_source_to_project(
        source_type: WorkItemSource,
        source_attributes: params[:work_item_source],
        saved_message: t('work_item_source.messages.source_created'),
        not_created_message: t('work_item_source.messages.source_not_created'),
        redirect_path: work_item_sources_path
    )

  end

  def update
    @work_item_source.display_name = params[:work_item_source][:display_name]
    if @work_item_source.save
      flash[:success] = t('work_item_source.messages.source_updated')
      redirect_to work_item_sources_path
    else
      render_crud_form(@work_item_source)
    end
  end

  # Finds the work item source by its ID or raises an error if it's not found.
  def find_source
    @source_server = @work_item_source = WorkItemSource.find(params[:id])
    raise MongoMapper::DocumentNotFound.new(t("work_item_source.errors.not_found")) unless @work_item_source
  end


  # Sets the active status on a work item source. The params[:new_status] must be one of "active" or "inactive".
  # Anything else does nothing.
  def active_status
    update_active_from_param(@work_item_source, params[:new_status])
    redirect_to work_item_sources_path
  end

  def canonicalize_server_params
    @server_params = params[:work_item_source]
  end
end
