#
# Copyright 2014 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#
require 'application_constants'

module Api::SchemasHelper
  require "amqp/extensions/rabbitmq"

  def create_url_for_schema(schema)
    parsed_schema = JSON.parse(schema)['custom_schema']
    escaped_schema_id = CGI::escape(parsed_schema['schema_id'])
    schema_version = parsed_schema['schema_version']

    if schema_version.present?
      url_for(controller: :schemas, :schema_id => escaped_schema_id, :schema_version => schema_version)
    else
      url_for(controller: :schemas, :schema_id => escaped_schema_id)
    end
  end


  def send_schema_to_queue(schema)
    rabbit_config = Rails.configuration.local_settings.load('rabbitmq')
    rabbit_uri = URI::parse(rabbit_config["uri"])
    rabbit_uname = rabbit_config["admin_user"]
    rabbit_password = rabbit_config["admin_password"]
    queue_name = CUSTOM_QUEUE_NAME

    EM.next_tick {

    amqp_failure_handler = Proc.new { logger.error( "- MQ AUTH ERROR LIKELY OCCURRED.") }
    AMQP.connect(:host => rabbit_uri.host, :username => rabbit_uname, :password => rabbit_password,
        :on_possible_authentication_failure => amqp_failure_handler) do |connection|

      channel = channel ||= AMQP::Channel.new(connection)
      channel.confirm_select
      queue = channel.queue(queue_name, durable: true)
      exchange = channel.default_exchange
      exchange.publish schema, :routing_key => queue.name

      channel.on_ack { connection.close }
    end
    }
  end
end
