#
# Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

# Helper methods for association trace controller
module AssociationTraceHelper

  # Finds the number of activities in the model
  # @param [Hash] data_model - the data model to find activities in
  # @return [Integer] the number of activities
  def calculate_activity_count(data_model)
    find_ids_by_recursion(data_model).flatten.uniq.reject{ |item| item == data_model[:id]}.count
  end

  # Reduces the data to just string IDs for finding the number of activities
  # @param [Hash] data_model - data model hash
  # @return [Array<String>] The IDs of this item and it's children (recursive)
  def find_ids_by_recursion(data_model)
    return [] unless data_model

    sum = [data_model[:id]]

    data_model[:children].each { | child | sum << find_ids_by_recursion(child) }

    sum.compact
  end

end