#
# Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

# Helper for commits.
module CommitHelper

  # Returns the repository name if the repository is defined. Otherwise return text indicating an unknown name.
  # @param [CommitServer] commit_server the repository
  # @return [String] the repository name or text indicating unknown name
  def repository_name(commit_server)
    commit_server ? commit_server.display_name : t("commit_server.unknown")
  end

  # Returns the repository host if the repository is defined. Otherwise, returns text indicating an unknown host.
  # @param [CommitServer] commit_server the repository
  # @return [String] the repository host or text indicating unknown host
  def repository_host(commit_server)
    commit_server ? commit_server.host : t("commit_server.unknown")
  end

  # Returns the repository type if the repository is defined. Otherwise, returns text indicating an unknown host.
  # @param [CommitServer] commit_server the repository
  # @return [String] the repository type or text indicating unknown host
  def repository_type(commit_server)
    commit_server ? commit_server.repository_type : t("commit_server.unknown")
  end

  # Splits the commit paths into fragments that can be wrapped in sections.
  # @param [String] path the path to split into pieces, delimited by /
  # @return [String] the string wrapped in divs that can be split by the browser
  def split_path(path)
    segments = path.split('/').reject(&:empty?)
    segments.map { |p| "<div class='path-fragment'>/#{h(p)}</div>"}.join.html_safe
  end

  # Creates a string containing the class of the status icon
  # @param [Change] change the change from the commit
  # @return a set of CSS class names to apply to the icon
  def file_status_icon_class(change)
    "change-icon icons-#{change.type}"
  end

end
