#
# Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

# Helper for the crud form.
module CrudFormHelper
  # Toggle light/dark rows.
  def alternate_rows
    cycle('dark-row', 'light-row')
  end

  # Return the activate button. The button can be enabled or disabled.
  # @return [String] the activate button link
  def activate_button(source_server)
    server_active = source_server.active?
    activation_buttons(source_server,
                       server_active ? t('form.activated') : t('form.activate'),
                       "button-right source-activate #{server_active ? 'selected' : ''}",
                       true)
  end

  # Create the deactivate button.  The button can be enabled or disabled.
  # @return [String] the deactivate button link
  def deactivate_button(source_server)
    server_active = source_server.active?
    activation_buttons(source_server,
                       server_active ? t('form.deactivate') : t('form.deactivated'),
                       "button-left source-deactivate #{server_active ? '' : 'selected'}",
                       false)
  end

  # Create the link to a source server with project context maintained
  # @return [String] the path to source server
  def path_to_source_server(source_server)
    path_opts = path_options_with_project_context(project: source_server.project)
    polymorphic_path(source_server, path_opts)
  end

  private

  # Create the link for the activate or deactivate source buttons.
  # @param [String] button_text the displayed text of the button
  # @param [String] html_class the css classes for the button
  # @param [Boolean] activate_source true if the button will activate the source, false otherwise
  # @return [String] the button link
  def activation_buttons(source_server, button_text, html_class, activate_source)
    id = activate_source ? 'activate' : 'deactivate'
    new_status = activate_source ? 'active' : 'inactive'
    button_class = "button #{html_class}"
    path_opts = path_options_with_project_context(project: source_server.project, action: 'active_status', new_status: new_status)
    link_to button_text, polymorphic_path(source_server, path_opts),
                method: :post,
                class: button_class,
                id: id
  end

  # Renders access and trigger logs if they exist in the given source server.
  def render_logs(source_server)
    has_trigger_status = source_server.respond_to?(:trigger_status) && !source_server.trigger_status.empty?

    locals = {}
    locals.merge!(access_status: source_server.access_status) if source_server.access_status
    locals.merge!(trigger_status: source_server.trigger_status) if has_trigger_status

    render partial: 'shared/logs', locals: locals unless locals.empty?
  end
end
