#
# Copyright 2014 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

module WorkItemHelper

  # creates an updated attribute-change list for use in Event History view
  # @param [Activity] work_item work item
  # @return [Array<Symbol>] array of attribute symbols changed in this instance
  def work_item_attribute_change_events(work_item, in_activity_stream = false)
    change_list = work_item.attribute_changes
    if in_activity_stream && display_status_icon_in_activity_stream?(work_item)
      change_list -= [:closed, :deleted] #those are supposed to be handled before this point
    end
    # move :status to the front of the list if present
    change_list.unshift(change_list.delete(:status)) if change_list.include? :status
    change_list
  end

  # Creates the summary header string for the work item, including a date and time.
  #
  # @param [WorkItem] work_item The work item to evaluate
  # @return [String] String to display as the header for the work item.
  def work_item_summary_header(work_item)
    # TODO: this method doesn't account for creation events, so it currently
    # declares everything is an update - even item creations.
    timestamp = ui_friendly_timestamp(work_item)
    if work_item.updated_by
      t('work_item.summary_with_updater',
        updated_by: work_item.updated_by,
        timestamp: timestamp)
    else
      t('work_item.summary_no_updater',
        timestamp: timestamp)
    end
  end

  def activity_history_icon_class(work_item, attribute_change, size=:small)
    show_status = ([:status, :deleted].include? attribute_change)
    "icons-status-#{(show_status) ? work_item.status_icon_class : 'bullet'}#{ "-small" if size == :small}"
  end


  def work_item_changed_attribute_description(work_item, attribute)
    case attribute
      when :deleted then t('activity.deleted')
      when :closed then changed_closed_state(work_item)
      when :status then t('activity.status_updated', status: work_item.display_status.upcase)
      when :assigned_to then changed_assignee(work_item)
      when :priority then t('activity.priority_updated', priority: work_item.priority.upcase)
      when :source_id then t('activity.source_updated', tracker: work_item.source.display_name.upcase)
      else t('activity.attribute_updated', attribute: attribute.to_s.humanize)
    end
  end

  def changed_assignee(work_item)
    if work_item.assigned_to.present?
      t('activity.assigned_to', assignee: work_item.assigned_to.upcase)
    else
      t('activity.unassigned')
    end
  end

  def changed_closed_state(work_item)
    work_item.closed ? t('activity.closed') : t('activity.reopened')
  end

  def work_item_status(work_item)
    changes = work_item.attribute_changes

    if work_item.deleted || changes.include?(:deleted)
      t('activity.deleted')
    elsif changes.include? :closed
      state_string_activity_stream(work_item, changed_closed_state(work_item))
    else
      state_string_activity_stream(work_item, work_item.closed ? t('activity.closed') : t('activity.open'))
    end
  end

  def state_string_activity_stream(work_item, state)
    friendly = work_item.display_status.titleize
    (friendly == state) ? friendly : "#{friendly} (#{state})"
  end

  def display_creation_time(work_item)
    [common_date_time(work_item.creation_time, :date_padded), common_date_time(work_item.creation_time, :time_with_tz)].join(' ')
  end
end
