#
# Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

class Ability
  include CanCan::Ability
  include CtfUserAuthorization

  # Adds a 'can' check for a given user to compare the provided permission with the project
  #
  # @param [User] user The user to check for project permission.
  # @param [Symbol] permission The permission to build a query around.
  def permit_by_project(user, permission)
    can(permission, Project) do |project|
      user_permission_cache = get_all_permissions_for_user(user)
      imply_permissions_and_get_project_list(user_permission_cache, permission, user).include? project.id.to_s
    end
  end

  # Fill out permissions for the user provided.  See https://github.com/ryanb/cancan/wiki/Defining-Abilities for more details.
  #
  # @param [User] user The user to establish permissions for.
  def initialize(user)
    user ||= User.new

    current_users_permissions = $redis_user_permissions.get(user.uid)
    user_permission_cache = JSON.parse(current_users_permissions) if current_users_permissions != nil
    if user_permission_cache
     [:pipeline_read, :pipeline_edit, :pipeline_create, :reporting_api].each { |perm| permit_by_project(user, perm) }
    end
  end
  
  # Returns permissions that cascade to include the specified permission.
  #
  # @param [Symbol] permission The permission being checked.
  # @return [Array of Symbols] The permissions that imply the specified permission (including itself).
  def self.permissions_implying(permission)
    case permission
    when :pipeline_read
      [:pipeline_read, :pipeline_edit, :pipeline_create]
    when :pipeline_edit
      [:pipeline_edit, :pipeline_create]
    else
      [permission]
    end
  end
  
end
