#
# Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

class ActivityFilter::Activities < ActivityFilter::Base

  validate :type_validations

  attr_accessor :filter_by_activity_types

  # Reads an array of activity types from a param
  # @param [Array] array of activity types
  def chosen_activity_types=(types)
    if types.is_a?Array
      if types.present?
        @chosen_activity_types = types
        verify_activity_types
      else
        @chosen_activity_types = nil
      end
    else
      @chosen_activity_types = :invalid
    end
  end
  #
  # # List of activity types
  # # @return [Array] activity types
  def chosen_activity_types
    if include_all_types
      nil
    else
      (@chosen_activity_types || []) - (@invalid_activity_types || [])
    end
  end

  # Attaches the list of activity types to the filter for use by the view
  # @param [String[]]
  def activity_types=(activity_types_ary)
    @activity_types = activity_types_ary
    verify_activity_types
    @activity_types
  end

  # # Accessor for activity types
  # # @return [Array] activity_types
  def activity_types
    @activity_types || []
  end

  # @return [Boolean] include all types
  def include_all_types
    !@filter_by_activity_types && (@chosen_activity_types.nil? || @chosen_activity_types == :invalid)
  end


  # Paginated list of activities returned by the filter
  # @param [Project] current project
  # @param [Hash] options for query, including pagination params
  # @return [Array of Activities] activities to display
  def activities(project, options = {})
    page_number     = options.delete(:page_number)
    page_item_count = options.delete(:page_item_count)
    project.most_recent(page_number, page_item_count, filter_params.merge(options))
  end
  
  # Build params for activities query
  # @return [Hash] options for activities query
  def filter_params
    {
      start:                  start ? start.to_time.beginning_of_day : nil,
      end:                    self.end ? self.end.to_time.end_of_day : nil,
      chosen_activity_types:  chosen_activity_types
    }
  end

  # Check for valid activity types
  def verify_activity_types
    @invalid_activity_types = @chosen_activity_types - activity_types if @chosen_activity_types.is_a?Array
  end

  # Type validations
  # Reports invalid type parameters to validator
  def type_validations
    if @chosen_activity_types
      if @chosen_activity_types == :invalid
        errors.add(:activity_types, :invalid)
      elsif @invalid_activity_types.present?
        errors.add(:activity_types, :bad_type)
      end
    elsif filter_by_activity_types
      errors.add(:activity_types, :empty)
    end
  end
  
end