class ApiSourceAdapter
  SOURCE_API_TO_MONGO_MAP = { 
    'Review' => 'ReviewServer',
    'WorkItem' => 'WorkItemSource',
    'XDS'  => 'CustomActivitySource',
    'Commit' => 'ExternalRepository',
    'CommitCTF' => 'CtfRepository',
    'Build' => 'BuildServer' 
   }

  attr_accessor :args, :errors

  def initialize(_args, current_project)
    self.args = _args.merge(project_id: current_project.id)
  end

  def type
    args[:_type] || args['_type']
  end

  def valid?
    #unused, leaving here for consistency
    valid_type? && native_source.valid?
  end

  def as_json(options={})
    if errors.any?
      errors.messages
    else 
      native_source.as_json(options)
    end
  end

  def save
    valid_type? && native_source.save
  end

  def native_source_class
    SOURCE_API_TO_MONGO_MAP[type].constantize
  end

  def valid_type?
    unless SOURCE_API_TO_MONGO_MAP.keys.include?(type)
      errors.add(:_type, "Type is invalid.")
      return false
    end
    true
  end
  private :valid_type?

  def native_source
    @native_source ||= native_source_class.new(params_curator.curated_params.merge(ActivitySource.generated_columns))
  end

  def params_curator
    @params_curator ||= ParamsCurator.new(native_source_class, args)
  end
  private :params_curator

  def errors
    @errors ||= Errors.new()
  end
  private :errors

  #reason for this abstraction: 
  #Keys are different from input params 
  #the requirement to not interfere with view based activity source interface, and
  #the unavaiability of external components like strong_parameters that working with mongomapper. 
  class ParamsCurator 
    attr_reader :args, :source_class

    def initialize(source_class, args)
      @args = args
      @source_class = source_class
    end

    def curated_params
      @curated_params ||= substitute(filter(args))
    end

    #filter out all attrs that are not permitted by the source class
    def filter(list)
      list.select { |key, val| source_class.permitted_attrs.include?(key.to_s) }
    end
    private :filter

    #substitute all the keys that have different native key
    def substitute(list)
      list.inject({}){ |memo, (key, val)| 
        memo[source_class.nativized_key(key.to_s)] = val 
        memo
      }
    end
    private :substitute

  end

end

