#
# Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

# Represents builds from a continuous integration server.
class Build < Activity
  VALID_STATUS_TYPES = %w(SUCCESS FAILURE UNSTABLE ABORTED)

  key :status_type, String
  key :status_name, String

  key :remote_id, String
  key :test_results_passed_count, String
  key :test_results_failed_count, String
  key :test_results_ignored_count, String
  key :test_results_url, String

  key :duration, Integer
  key :build_url, String
  timestamps!

  activity_source :build_server

  # provided revisions are not persisted to the database, instead they are used at message parse time to
  # associate to existing commits or create phantom commits [wiki7215]
  attr_accessor :provided_revisions

  alias :display_number :remote_id

  attr_accessible *(keys.keys.map(&:to_sym) + [:provided_revisions])
  validates_presence_of :status_name, :status_type, :remote_id, :duration, :time_created, :build_url


  # @return [boolean] Whether or not there are test counts in the build.
  #
  def has_test_counts?
    ([test_results_passed_count, test_results_failed_count, test_results_ignored_count].compact.length > 0)
  end

end
