#
# Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#


class BuildSummary < ActivitySummary

  key :test_results_passed_count, String
  key :test_results_failed_count, String
  key :test_results_ignored_count, String
  key :test_results_url, String

  key :status_type, String
  key :status_name, String  # a user friendly display name, it seems we don't use it anywhere in our code.

  key :duration, Integer
  key :build_url, String

  key :remote_id, String

  alias :display_number :remote_id

  attr_accessible *(keys.keys.map(&:to_sym))

  alias :associated_commit_summaries :activity_summaries


  # @return [boolean] Whether or not there are test counts in the build.
  #

  def has_test_counts?
    ([test_results_passed_count, test_results_failed_count, test_results_ignored_count].compact.length > 0)
  end

  # creates  a build summary from a build object
  # @param [Build] a build object
  # @return [BuildSummary]
  def self.create_or_update(build)
    BuildSummary.find_or_initialize_by_remote_id_and_source_id(build.remote_id, build.source_id).tap do |build_summary|

      unless build_summary.persisted? && build_summary.settled?
        build_summary.created_by = build.created_by
        build_summary.time_created = build.time_created
      end

      build_summary.status_type = build.status_type
      build_summary.status_name = build.status_name
      build_summary.test_results_passed_count = build.test_results_passed_count
      build_summary.test_results_failed_count = build.test_results_failed_count
      build_summary.test_results_ignored_count = build.test_results_ignored_count
      build_summary.test_results_url = build.test_results_url
      build_summary.duration = build.duration
      build_summary.build_url = build.build_url
      build_summary.activities << build
      build_summary.settled = true
      build_summary.activity_summaries << build_summary.commit_summaries_from_revisions(build.provided_revisions)
      build_summary.save
      build_summary.associate_to_activity_summaries!(build_summary.activity_summaries)
    end
  end
  add_method_tracer :create_or_update

  # finds commit summaries that match the revisions reported by the build server
  # @param [Array<IncludedRevision>] provided_revisions revisions from the build queue message
  # @return [Array<CommitSummary>]
  def commit_summaries_from_revisions(provided_revisions)
    if source.commit_server
      provided_rev_numbers = provided_revisions.to_a.map {|r| r.revision}
      existing_summaries = source.commit_server.activity_summaries
      existing_rev_numbers = existing_summaries.where(:revision_number => {:$in => provided_rev_numbers}).all.map {|s| s.revision_number}
      unknown_rev_numbers = provided_rev_numbers - existing_rev_numbers
      unknown_rev_numbers.map do |revision|
        CommitSummary.find_or_create revision, source.commit_server
      end
      CommitSummary.where(:revision_number => {:$in => provided_rev_numbers}).to_a
    else
      []
    end
  end
end
