#
# Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

# Represents commits to a source control repository.
class Commit < Activity
  embedded_callbacks_off

  key :message, String
  key :revision_number, String
  key :branch_name
  many :path_changes
  one :ctf_item
  timestamps!

  activity_source :commit_server

  # this check is performed at the application level, so there might be concurrency issues if several clients try to write
  # the same data simultaneously.  It would be nice to add a unique constraint to a composite index in MongoDB.
  validates :revision_number, :uniqueness => {:scope => [:source_id]} #FIXME: long running

  attr_accessible *(keys.keys.map(&:to_sym) + [:path_changes, :ctf_item])

  delegate :path_changes_sorted, to: :activity_summary

  # Return the display 'number' for this commit
  # @return [String] the display number
  def display_number
    revision_number.size > 20 ? revision_number[0,7] : revision_number
  end
end
