#
# Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

class Counters
  include MongoMapper::Document

  key :name, String
  key :count, Integer, default: (OpaqueID::RADIX ** 3) + 1  # At least 4 characters

  validates_uniqueness_of :name

  # Atomically increment and return this Counter's' count.
  # @param [integer] quantity how many numbers to return (used for bulk creation)
  # @return the new count
  def increment_and_fetch(quantity = 1)
    Counters.collection.find_and_modify(query: {name: self.name}, update: {:$inc => {:count => quantity}}, new: true)['count']
  end

  alias_method :next, :increment_and_fetch
end
