#
# Copyright 2014 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#
class RequiredFieldValuesValidator < ActiveModel::EachValidator
  def validate_each(record, attribute, value)
    CustomSchema::REQUIRED_FIELDS.each do |required_field|
      unless value && value.include?(required_field)
        record.errors[attribute] << I18n.t('custom_activity_source.errors.attribute_missing', attribute: "#{attribute}", required_field: "#{required_field}")
      end
    end
  end
end

class SuperSetFieldValuesValidator < ActiveModel::EachValidator
  def validate_each(record, attribute, value)
    value.nil? || value.each do | value |
      unless record.fields.present? && record.fields.include?(value)
        record.errors[attribute] << I18n.t('custom_activity_source.errors.extra_value', attribute: "#{attribute}", value: "#{value}")
      end
    end
  end
end

class CustomSchema
  include MongoMapper::Document

  key :event_time, Time
  key :name, String
  key :schema_id, String
  key :schema_version, Fixnum
  key :fields, Array
  key :important, Array
  key :parsed, Array
  key :required, Array
  key :hidden, Array

  belongs_to :custom_schema_store

  validates_presence_of :event_time, :schema_id, :name, :fields, :required, :schema_version
  #fields must include the following elements, event_time, remote_id, schema_id, schema_version
  validates_with RequiredFieldValuesValidator, attributes: [:fields, :required]
  # required, important and parseable fields must contain values already specified in fields
  validates_with SuperSetFieldValuesValidator, attributes: [:required, :important, :parsed, :hidden]

  REQUIRED_FIELDS =  ['event_time', 'remote_id', 'schema_id', 'schema_version']

  # checks that all required fields are present and message does not pass more data than promised in schema.
  def valid_message?(message)
    msg_fields = message.keys.map{|k| k.to_s}
    non_empty_fields = message.map { |k,v| k.to_s if v.present? }.compact
    (required - non_empty_fields).size == 0 &&
        (msg_fields - fields).size == 0
  end
end
