#
# Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#


# Represents an instance of a custom schema
class CustomSchemaStore
  include MongoMapper::Document

  key :schema_id, String
  has_many :custom_activity_sources
  has_many :custom_schemas


  def current_schema
    custom_schemas.sort_by(&:schema_version)[-1]
  end

  def name
    current_schema.name
  end

  def has_version?(version)
    custom_schemas.map(&:schema_version).include? version
  end


  # Checks for the presence of CustomSchemaStore for a specific schema_id. If it does not exist, one is created and the
  # schema is added to it. If store exists, check for schema with specific version. If not found, create schema,
  # else skip.
  def self.build_schema(message)
    find_or_initialize_by_schema_id(message['schema_id']).tap do |schema_store|
      unless schema_store.has_version?(message['schema_version'])
        schema = CustomSchema.new(message)
        raise MessageProcessingError, I18n.t("custom_activity_source.messages.could_not_save_schema",
                                             errors: schema.errors.messages.values.to_sentence) unless schema.valid?
        schema_store.custom_schemas << schema
        schema_store.save
      end
    end
  end
end
