#
# Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

# Configuration for CTF projects that are connected to Orchestrate.
class TeamForge::ProjectConfiguration
  include MongoMapper::Document

  key :server_url, String
  key :application_id, String
  key :ctf_project_id, String
  key :project_name, String
  key :project_url, String
  key :user_id, String
  key :template, Boolean
  key :link_plug_id, String
  key :app_prefix, String
  key :active, Boolean, default: true

  belongs_to :project

  validates :project_name, presence: true
  validates :ctf_project_id, presence: true

  after_save :check_inactive

  # Mark the corresponding project record inactive whenever this record is marked inactive.
  def check_inactive
    self.project.update_attributes(active: false) unless self.active
  rescue NoMethodError => e
    logger.warn("Unable to delete Project #{self.project_name} (possibly deleted?): #{e.to_s}")
  end

  # Intercept the default query, and attach the 'active' condition to it.  Forces only active records to be
  # returned in queries.  To access inactive records, pass 'active: false'.
  #
  # TODO: in this implementation, passing 'active: false' results in returning records which are either true
  # OR false (i.e., all records) since criteria are successively merged.
  #
  # NOTE: This override affects result sets which are produced by query, and NOT those produced from the
  # collection as a whole.  E.g.:
  #
  # Class.all(active: false) - Only the false records; 'all' is acting like a filter on the collection.  No query is created.
  # Class.where(active: false).all - 'where' is merging the criteria; 'all' is acting as a filter on the query results.
  def self.query(options={})
    super.tap { |query| query[:active] = true unless options.key?(:active) }
  end

  # retrieves the link_plug_id from DB if it exists in there , otherwise fetches it from CTF and saves it in DB
  # param [String] session_id, CTF session id
  #
  # TODO: rename this method so that it's clear what it's doing (i.e., a save operation).
  def get_link_plug_id(session_id)
    link_id = link_plug_id
    unless link_id
      pluggable =  CtfService.create_teamforge_session(session_id).pluggable
      base_url = Rails.configuration.teamforge[:iaf][:orc_url]
      project_path = "projects." + project_url
      link_id = pluggable.get_link_plug_id(project_path: project_path, base_url: base_url)
      self.link_plug_id = link_id
      self.save
    end
    link_id
  end

end
