#
# Copyright 2015 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#
class Tool
  include MongoMapper::Document

  MAX_NAME_LENGTH = 100

  key :name, String
  key :service_name, String
  key :active, Boolean, default: true

  belongs_to :project
  many :sources, class: ActivitySource
  attr_accessor :source_ids

  validates_presence_of :service_name
  validates_presence_of :name

  validate :validate_sources

  after_save :update_sources
  after_update :update_sources

  scope :by_project, ->(project_id) {where( project_id: project_id) }
  scope :by_name, ->(name) {where( name: name) }


  def as_json(options = {})
    # Get the API version
    #
    # Format:
    # {:prefixes=>["api/v1/trackers", "api/base"]}
    #
    api_version = 'v1'
    api_version = options[:prefixes][0].split('/')[1] if options[:prefixes].present?
    "#{api_version.camelize}::ToolSerializer".constantize.new(self).serialize
  end

  def update_sources
    if self.source_ids
      new_sources = ActivitySource.where(:id => self.source_ids).all
      self.sources.replace(new_sources) 
    end
  end

  def validate_sources
    new_sources = ActivitySource.where(:id => self.source_ids).all

    if self.source_ids && new_sources.count != self.source_ids.count
      errors.add(:sources, "Some of the sources were not found.")
    end

    if new_sources.select {|source| source.project != project}.count > 0
      errors.add(:sources, I18n.t('project.tool.sources_validation'))
    end
  end
  private :validate_sources

  def self.name_unique?(project, name)
    Tool.by_project(project.id).by_name(name).empty?
  end
end
