#
# Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#
# represents a work-item tracker
class Tracker
  include MongoMapper::Document

  embedded_callbacks_off

  key :remote_id, String
  key :name, String
  key :key, String
  key :regex, String
  key :icon, String   # value is a url
  key :active, Boolean, default: true

  belongs_to :work_item_source
  many :statuses

  validates_presence_of :remote_id, :name, :regex
  validates_associated :statuses
  validates :work_item_source_id, uniqueness: { case_sensitive: false }, allow_blank: true
  validate :validate_work_item_association

  def self.active
    where(active: true)
  end

  def as_json(options = {})
    # Get the API version
    #
    # Format:
    # {:prefixes=>["api/v1/trackers", "api/base"]}
    #
    api_version = 'v1'
    api_version = options[:prefixes][0].split('/')[1] if options[:prefixes].present?
    "#{api_version.camelize}::TrackerSerializer".constantize.new(self).serialize
  end

  private

  # the work_item_source_id must be a foreign key to a work_item_source if it is present
  def validate_work_item_association
    return if work_item_source_id.blank?

    unless WorkItemSource.find(work_item_source_id.to_s)
      errors.add(:work_item_source_id, 'the work_source_id is not associated with any work item source')
    end
  end

end
