#
# Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

class WorkItem < Activity
  include ChangeHistory

  timestamps!

  key :remote_id, String
  key :summary, String
  key :description, String
  key :remote_status_id, String
  key :closed, Boolean
  key :deleted, Boolean
  key :priority, String
  key :assigned_to, String
  key :creation_time, Time
  key :tags, Array
  key :attribute_changes, Array
  key :ancestor_id, String
  key :updated_by, String

  # These are the attributes, which, if modified, should be captured by an action for 
  # the activity history so we can display activity details.
  SIGNIFICANT_ATTRIBUTES = [:summary, :description, :closed, :deleted, :priority, :assigned_to, :tags,
                            :source_id, :status, :created_by, :creation_time]

  activity_source :work_item_source

  attr_accessible *(keys.keys.map(&:to_sym))

  validates_presence_of :summary, :remote_id, :remote_status_id

  alias :display_number :remote_id

  # Implements display_status to display the Work item trackers status
  include WorkItemStatus

  alias :status :remote_status_id

  before_create :calculate_attribute_changes, :validate_significant_delta
  before_save :set_assignee

  # Override of Activity.status_icon_class, since work item uses booleans to map to an Orchestrate status
  #
  # @return |String| the status, based on the closed and deleted booleans
  def status_icon_class
    if deleted
      'deleted'
    elsif closed
      'closed'
    else
      'open'
    end
  end

  # Returns the link-back url of this work item
  # @return [String] the link-back url
  def url
    URI.join(work_item_source.url + '/', remote_id).to_s
  end

  # Set 'nobody' as a value for assigned_to is empty string.
  # due to Webhook events
  def set_assignee
    self.assigned_to = 'nobody' if self.assigned_to.blank?
  end

end
