#
# Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

# A WorkflowRule can be used to gate various transitions
# An example is preventing a commit because the commit message
# does not match the regex.  The Workflow rule for that would
# look like this:
#
#  {collection_name: 'commits',
#   field_name: 'commit_message',
#   regex: %r{<your regex>},
#   error_message: '<the error message if the condition fails>'}
class WorkflowRule
  include MongoMapper::Document

  key :collection_name
  key :field_name
  key :regex
  key :error_message

  def can_commit(commit_server, commit_message)
    allow_commit = true
    if commit_server.try(:commit_gate)
      allow_commit = commit_message =~ regex
    end
    allow_commit
  end
end
