module V1
  class ActivitySourceSerializer
    attr_reader :object

    MONGO_TO_API_SOURCE_TYPE_MAP = ApiSourceAdapter::SOURCE_API_TO_MONGO_MAP.invert

    def initialize(src)
      @object = src
    end

    def serialize(options = {})
      queue_server = ActivitySourceSerializer.queue_server
      if object.errors.empty?
        {
          id: object.id,
          _type:  MONGO_TO_API_SOURCE_TYPE_MAP[object._type],
          tool_id: object.tool_id,
          active: object.active,
          display_name: object.display_name,
          queue_username: object.queue_username,
          queue_password: object.queue_password,
          queue_server: queue_server,
          source_association_key: object.source_association_key,
        }
      else
        object.errors.messages
      end
    end

    def self.queue_server
      unless defined?(@queue_server)
        begin
          uri = URI.parse(Rails.configuration.rabbitmq[:uri])
          # Remove user and password just in case
          uri.user = nil
          uri.password = nil
          @queue_server = "#{uri.to_s}"
        rescue URI::InvalidURIError => uri_error
          Rails.logger.warn "RabbitMQ URL is invalid"
        end
      end
      @queue_server
    end

    def self.substitute_keys(hash, dictionary)
      hash.inject({}) { |mem, (key, val) |
        new_key = dictionary[key] ||
                  dictionary[key.to_sym] ||
                  dictionary[key.to_s] || key

        mem[new_key] = val
        mem
      }
    end

  end
end