#################################################################################
#  packet_injector.rb: inserts attributes into a packet in the form required by
#  various legacy system collections (work_item, commit etc.)
#  Uses the singleton pattern
#
#
=begin


    #############################################################
        WORK_ITEM JSON
    #############################################################

    {
        "api_version": "1",
        "source_association_key" : "<%= workitem_association_key %>",
        "work_item" : {
            "created_by": "itembot!",
    <% if updated_by %>        "updated_by": "<%= updated_by %>",
    <% end -%>
            "id": "<%= workitem_id %>",
            "tracker_id": "autotracker-0",
            "event_time": "<%= event_at %>",
            "summary": "<%= summary %>",
            "description": "Autogenerated workitem description",
            "status_id": "<%= status = [1,2,3].sample %>",
            "closed": <%= status == 3%>,
            "deleted": false,
            "priority": "<%= priority %>",
            "assigned_to": "<%= assigned_to %>",
            "creation_time": "2014-03-29T00:26:20.091Z",
            "tags": <%= tags %>
        }
    }

Required Attributes
   api_version (string), source_association_key(string), id (string), event_time(string), summary(string),
   status_id(string), tracker_id(string)


    ########################################################
    *   COMMIT EVENT JSON
    ########################################################

    {
        "api_version": "1",
        "source_association_key" : "<%= commit_association_key %>",
        "commit_data" : {
            "repo_id": "150e8951-19e1-4066-8b74-601026dd6cde",
            "revision_id": "<%= revision %>",
            "branch_name": "master",
            "type": "post_commit",
            "message": "Commit <%= action %> the <%= object %> [<%= review_revision || "orc-277" %>][<%= workitem_id %>]",
            "changes": [
                { "path": "/source/file/1", "action": "modified" },
                { "path": "/source/file/4", "action": "modified" }
            ],
            "event_time": "<%= event_at %>",
            "created_by": "commitbot!"
        }
    }


Required Attributes
   api_version (string), source_association_key(string). revision_id (UUID), type(string), event_time(string),
   created_by(string), path(string), action(string)


################################################################
# BUILD EVENT
################################################################

    {
        "source_association_key": "<%= build_association_key %>",
        "api_version": "1",
        "build_data": {
            "remote_id": "<%= build_number %>",
            "created_by": "commitbot",
    <% if failure -%>
            "status": {
                "type": "FAILURE",
                "name": "Failure"},
    <% else -%>
            "status": {
                "type": "SUCCESS",
                "name": "SUCCESS"},
    <% end -%>
            "test_results": {
                 "passed_count": "<%= failure ? 10 : 21 %>",
                 "failed_count": "<%= failure ? 11 : 0 %>",
                 "ignored_count": "0",
                 "url": "http://bar.com"},
            "duration": "20",
            "event_time": "<%= built_at %>",
            "build_url": "http://example.com",
            "revisions": [<% -%>
    <% revisions.each.with_index do |built_revision,index| -%>
            { "revision": "<%= built_revision %>",
            "repository_type": "svn",
            "repository_url": "https://svnserver/project"
          }<%= ",\n" if (index+1) != revisions.length -%>
    <% end -%>]
        }
    }

Required Attributes
api_version(string), source_association_key(string), build_data, remote_id(string), duration(string),
event_time(string), build_url(string), status(string), type(string: SUCCESS,FAILURE,UNSTABLE,ABORTED),
name(string), revision(string), repository_url(string)



################################################################
# REVIEW EVENT
################################################################


    {
        "source_association_key": "<%= build_association_key %>",
        "api_version": "1",
        "request_data": {
            "remote_id": "<%= build_number %>",
            "created_by": "commitbot",
    <% if failure -%>
            "status": {
                "type": "FAILURE",
                "name": "Failure"},
    <% else -%>
            "status": {
                "type": "SUCCESS",
                "name": "SUCCESS"},
    <% end -%>
            "test_results": {
                 "passed_count": "<%= failure ? 10 : 21 %>",
                 "failed_count": "<%= failure ? 11 : 0 %>",
                 "ignored_count": "0",
                 "url": "http://bar.com"},
            "duration": "20",
            "event_time": "<%= built_at %>",
            "build_url": "http://example.com",
            "revisions": [<% -%>
    <% revisions.each.with_index do |built_revision,index| -%>
            { "revision": "<%= built_revision %>",
            "repository_type": "svn",
            "repository_url": "https://svnserver/project"
          }<%= ",\n" if (index+1) != revisions.length -%>
    <% end -%>]
        }
    }

Required Fields: api_version(string), source_association_key(string), request_data(string), remote_id(string),
status(object), event_time(string), created_by(string), type("open,submitted,rejected,discarded,other"), name(string)
file(string)username(string)


    #########################################
    # custom schema
    ########################################

{
    "api_version": "1",
    "source_association_key" : "<%= custom_event_association_key %>",
    "custom_schema" : {
        "event_time": "<%= event_at %>",
        "name": "<%= custom_schema_name %> <%= custom_schema_id %>",
        "schema_id": "<%= custom_schema_id_name %><%= custom_schema_id %>",
        "schema_version": <%= schema_version.to_i %>,
        "fields": [
            "event_time",
            "schema_id",
            "remote_id",
            "schema_version",
            "summary",
            "description",
            "associated_remote_id",
            "deploy_url",
            "deploy_message",
            "stage",
            "created_by",
            "status_name",
            "status_type"

        ],
        "required": ["remote_id", "event_time", "schema_id", "schema_version"],
        "important": ["status_name"],
        "parsed": ["deploy_message", "deploy_url"]
    }
}


    #########################################
    # custom schema event
    ########################################

{
    "api_version": "1",
    "source_association_key": "<%= custom_event_association_key %>",
    "custom_data": {
        "event_time": "<%= event_at %>",
        "schema_id": "<%= custom_schema_id_name %><%= custom_schema_id %>",
        "remote_id": "<%= custom_remote_id %>",
        "schema_version": <%= schema_version.to_i %>,
        "summary": "<%= action %> <%= failure ? "deployment from" : "to"%> <%= stage %>",
        "associated_remote_id": "<%= workitem_id %>",
        "deploy_url": "http://www.example.com",
        "deploy_message": "<%= stage %> environment is <%= failure ? "removing" : "getting"%> the latest code from <%= workitem_id %>",
        "stage": "<%= stage %>",
        "created_by": "CustomBot!",
        "status_type": "<%= failure ? "failed" : "success" %>",
        "status_name": "<%= action %>"
    }
}

Required Fields:
api_version(string), source_association_key(string), schema_id, schema_version, event_time, remote_id


=end
#################################################################################

require 'singleton'

class PacketInjector
  include Singleton

  ###########################################################################
  # interface: receives a data packet as a ruby hash object, Creates a new
  # object destined for the legacy system which conforms to the topology
  # requirements imposed by the legacy Activities collection
  # receives (i)   the incoming data packet (hash)
  #          (ii)  the packet type (string)
  #          (iii) a packet_mapper (hash)
  ###########################################################################

  def interface(data_packet, packet_type, packet_mapper)
    ServiceLog.stack_trace('PacketInjection::interface') if Rails.application.config.STACK_TRACE_PACKET_INJECTOR

    @incoming_packet = data_packet
    @packet_type     = packet_type

    if @packet_type.blank?
      ServiceLog.log('PacketInjector::interface: interface did not receive a packet type parameter')
      return false
    end


    ###########################################################
    # Get the Packet Map for transformation of the data
    # packet. Then get the target in the legacy system
    ###########################################################

    @packet_map = packet_mapper
    if @packet_map.blank?
      ServiceLog.log('PacketInjector::interface: this packet_type does not have the packet map: ' + packet_type)
      return false
    end

    ############################################################
    # get the tracker and activity source objects for this
    # packet
    ############################################################

    return false if populate_with_tracker_and_source(data_packet, packet_mapper).blank?


    ###########################################################
    # construct a container for a conforming data packet
    # to be passed to the legacy system for insertion
    ############################################################

    return false unless construct_packet_envelope


    ####################################################
    # merge the incoming packet with the target packet
    ####################################################

    begin
      @target_packet.merge!(data_packet)
    rescue => e
      ServiceLog.log('PacketInjector::interface : cannot construct target packet :' + e.message)
      return false
    end


    #############################################
    # insert conforming fields for the legacy
    # activity collection
    #############################################

    return false unless inject_api_version
    return false unless inject_event_time


    #################################################
    # insert_target_fields
    #################################################

    return false unless insert_target_fields



    #################################################
    # return the work item source reference and the
    # constructed packet
    #################################################

    [@incoming_packet['source'], @target_packet]

  end


  private


  ##########################################################################
  # construct_packet_envelope: constructs a container for the data packet
  # which is to be persisted into a legacy collection.
  # The type of container constructed depends on the activity source type
  ##########################################################################

  def construct_packet_envelope
    ServiceLog.stack_trace('PacketInjection::construct_container') if Rails.application.config.STACK_TRACE_PACKET_INJECTOR

    type = @incoming_packet['source']['_type']
    
    case type
      when 'WorkItemSource'
        inner_container = {}
      
        # We need to capture whether the Payload is delete event or not for Work Item.
        # If it is, we simply add a delete flag inside of the container
        # so, we can capture the delete event.
        unless @packet_map['delete_events'].blank?
          inner_container['deleted'] = @packet_map['delete_events'].split(',').include?(@incoming_packet[@packet_map[@packet_type]['event_name']]) ? 'true' : 'false'
        end

        # We need to capture whether the Payload is done event or not for Work Item.
        # If it is, we simply add a close flag inside of the container
        # so, we can capture the close event.
        unless @packet_map['close_events'].blank?
          inner_container['closed'] = @packet_map['close_events'].split(',').include?(@incoming_packet[@packet_map[@packet_type]['status_category']]) ? 'true' : 'false'
        end

        @target_packet = {
            'work_item' => inner_container
        }
        @container_key = 'work_item'

      when 'ExternalRepository', 'CommitServer', 'CtfRepository'

         @target_packet = {
           'commit_data' => {}
        }
      @container_key = 'commit_data'

      when 'BuildServer'
        @target_packet = {
            'request_data' => {}
        }
        @container_key = 'request_data'

      when 'ReviewServer'
        @target_packet = {
            'request_data' => {}
        }
        @container_key = 'build_data'

      when 'CustomActivitySource'
        @target_packet = {
            'custom_data' => {}
        }
        @container_key = 'custom_data'

      else
        ServiceLog.log('PacketInjector::construct_container : cannot construct a container for this packet type :' + @packet_type)
        return false
    end

    true
  end


  ##########################################################################
  # populate_with_tracker_and_source: gets the tracker and  activity
  # source objects and places references to these in the received
  # packet. The incoming packet is used as a volatile source.
  # parameters: data packet
  ###########################################################################

  def populate_with_tracker_and_source(data_packet, packet_mapper)
    ServiceLog.stack_trace('PacketInjection::populate_with_tracker_and_source') if Rails.application.config.STACK_TRACE_PACKET_INJECTOR

    ##########################################
    # verify that an activity source exists
    # for the data packet
    ##########################################

    begin

      source = ActivitySource.find_by_source_association_key(@incoming_packet['source_association_key'])

      if source.blank?
        ServiceLog.log('PacketInjector::populate_with_tracker_and_source : activity source not found for this data packet :' + @incoming_packet.to_json)
        return false
      end
    rescue => e
      ServiceLog.log('PacketInjector::populate_with_tracker_and_source : exception finding activity source :' + e.message)
      return false
    end


    ########################################################
    # verify that a tracker exists for the  activity source
    ########################################################

    if source._type == "WorkItemSource"
      tracker = update_work_item_related_objects(source, data_packet, packet_mapper)
      return false unless tracker
    end

    # save the object references into the incoming data packet
    @incoming_packet['source']  = source

    true
  end


  ######################################################################
  # inject_api_version: if the version is not specified, a hard-coded
  # version is used
  ######################################################################

  def inject_api_version
    ServiceLog.stack_trace('PacketInjection::inject_api_version') if Rails.application.config.STACK_TRACE_PACKET_INJECTOR

    # insert the api version
    begin
      if @packet_map[@packet_type].present? && @packet_map[@packet_type]['api_version'].present?
        @target_packet['api_version'] = @packet_map['api_version']
      else
        @target_packet['api_version'] = '1'
      end
    rescue => e
      ServiceLog.log('PacketInjector::api_version : api version exception: ' + e.message)
      return false
    end

    true
  end


  ##########################################################################
  # inject_event_time
  # The target packet destined for the legacy collection requires an
  # event_time field which is representable in rfc3339 format
  # if the event time is not specified in the packet map, the current time
  # is inserted
  ##########################################################################

  def inject_event_time
    ServiceLog.stack_trace('PacketInjection::inject_event_time') if Rails.application.config.STACK_TRACE_PACKET_INJECTOR

    begin
      @target_packet[@container_key]['event_time'] = @packet_map[@packet_type]['event_time']
      if @target_packet[@container_key]['event_time'].blank?
        @target_packet[@container_key]['event_time'] = Time.now.to_datetime
      end

    rescue

      ServiceLog.log('PacketInjector::inject_event_time : using synthetic event time; event_time was not specified in the packet map for: ' + @packet_type)
      @target_packet[@container_key]['event_time'] = Time.now.to_datetime
    end

    #-------------------------------------------
    # convert the event time to RFC3339 Format
    #-------------------------------------------
    begin
      @target_packet[@container_key]['event_time'] = @target_packet[@container_key]['event_time'].rfc3339
    rescue => e
      ServiceLog.log('PacketInjector::inject_event_time : unable to convert event time to rfc3339 format - ' + e.message)
      return false
    end

    true
  end


  #########################################################################
  # insert_target_fields: inserts attributes into a packet destined for
  # the legacy system, depending upon the packet map and the
  # legacy packet topology (workitem, review, build, etc.)
  #########################################################################
  
  def insert_target_fields

    ServiceLog.stack_trace('PacketInjection::insert_target_fields') if Rails.application.config.STACK_TRACE_PACKET_INJECTOR

    if @packet_map[@packet_type].blank?
      ServiceLog.log('packet_injection: insert_target_fields: packet_map[packet_type] hash does not exist' )
      return false
    end

    keys_array = @packet_map[@packet_type].values

    keys_array.each do |key_value|

      begin
        field_value = @incoming_packet[key_value]
        @target_packet[@container_key][@packet_map[@packet_type].key(key_value)] = field_value
      rescue
        ServiceLog.log('packet_injection: insert_target_fields: ' + key_value + " in the packet map cannot be mapped in the incoming packet" )
        return false
      end
    end

    true
  end

  #########################################################################
  # update_work_item_related_objects: updates a tracker object / work item source from a data packet
  # or updates an existing tracker / work item source object.
  # Persists the tracker object into the legacy system
  # receives: (i)   an activity source
  #           (ii)  data object
  #           (iii) packet_mapper
  # returns a tracker message or empty hash
  ########################################################################

  def update_work_item_related_objects(source, data_packet, packet_mapper)
    ServiceLog.stack_trace('PacketInjection::update_work_item_related_objects') if Rails.application.config.STACK_TRACE_PACKET_INJECTOR

    # common variables
    mapper = packet_mapper[@packet_type]
    status_name = data_packet[mapper['status_name']]
    status_id = data_packet[mapper['status_id']]
    remote_id = data_packet[mapper['remote_id']]
    tracker_name = data_packet[mapper['tracker_name']]
    tracker_key = data_packet[mapper['tracker_key']]
    tracker_icon = data_packet[mapper['tracker_url']]
    tracker_changed = false

    # Based on the Payload, we need to get updated Go URL in work item source.
    go_url = get_updated_go_url(source, data_packet, packet_mapper)

    world_state_message = { 'source_association_key' => source.source_association_key,
                            'api_version' => '1',
                            'workitem_settings' => { 'tracker' => {},
                                                     'url' => go_url['url'],
                                                     'event_time' => Time.now.to_datetime.rfc3339
                                                   }
                          }
    tracker_message = {}
    statuses = []

    if tracker = Tracker.find_by_work_item_source_id(source.id)
      statuses_object = {}
      tracker.statuses.each { | s | statuses_object[s.remote_id] = s.name }

      # if status_is does not exist, add as a new entry
      if statuses_object[status_id].nil?
        statuses << { 'name' => status_name, 'id' => status_id }
        tracker_changed = true
      end

      # get and modify existing tracker statuses
      tracker.statuses.each do | ts |
        t_remote_id = ts.remote_id
        if status_id == t_remote_id
          if statuses_object[status_id] != status_name
            t_name = status_name
            tracker_changed = true
          else
            t_name = ts.name
          end
        else
          t_name = ts.name
        end

        statuses << { 'name' => t_name, 'id' => t_remote_id }
      end
      tracker_changed = tracker_changed || tracker.key != tracker_key || tracker.name != tracker_name || tracker.icon != tracker_icon
    else
      statuses = [ { 'name' => status_name, 'id' => status_id } ]
      tracker_changed = true
    end


    # Send world state message if any field is updated or new Tracker
    if tracker_changed || go_url['changed']
      tracker_message['regex'] = "#{tracker_key}-?\\d+"
      tracker_message['name'] = tracker_name
      tracker_message['icon'] = tracker_icon.blank? ? '' : tracker_icon.gsub("/", "\/")
      tracker_message['id'] = remote_id
      tracker_message['key'] = tracker_key
      tracker_message['statuses'] = statuses
    end

    # save the changes if tracker gets changed
    begin
      unless tracker_message.empty?
        world_state_message['workitem_settings']['tracker'] = tracker_message
        RabbitConnector.instance.interface(source.queue_username, source.queue_password, 'work_items', world_state_message)
      end
    rescue => e
      ServiceLog.log('packet_injection: update_work_item_related_objects: unable to update / create a Tracker' + e.message)
      return false
    end

    tracker_message
  end


  #########################################################################
  # update_work_item_related_objects: Get Updated GO URL
  # with work item source based on Payload.
  #
  # receives: (i)   an work item source
  #           (ii)  data object
  #           (iii) packet_mapper
  ########################################################################

  def get_updated_go_url(source, data_packet, packet_mapper)

    go_url = packet_mapper[@packet_type]['go_url']
    changed = false

    unless go_url.blank?
      url = data_packet[go_url]
      uri = URI(url)
      scheme = uri.scheme.strip.downcase if uri.scheme
      hostname = uri.host.strip.downcase if uri.host
      go_url_base = "#{scheme}://#{hostname}/"

      # add a go url path
      path = packet_mapper['go_url_path']
      unless path.blank?
        go_url = "#{go_url_base}#{path}/"
      end

      if source.url.blank? || source.url != go_url
        changed = true
      end
    end

    { 'url' => go_url,
      'changed' => changed }
  end
  
end
