#
# Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

json.key_format! :camelize => :lower

json.totalItems @activities.length
json.totalPages @activities.total_pages if @activities.respond_to? :total_pages
json.currentPage @activities.current_page if @activities.respond_to? :current_page

json.items @activities do |activity|
  # This comes first so that anything can override it, but the original message's fields will be used if not overridden.
  if activity.respond_to? :raw
    raw = JSON.parse(activity.raw)
    json.extract!(raw, *(raw.keys))
  end

  json.title activity_display_name(activity)
  json.generator do
    source = if activity.source.respond_to? :uri
               activity.source.uri
             elsif activity.source.respond_to?(:repository_uri) && activity.source.repository_uri.blank?
               activity.source.project_uri
             elsif activity.source.respond_to? :repository_uri
               activity.source.repository_uri
             else
               'external'
             end
    json.url source
  end
  json.provider do
    json.url root_url
  end

  unless activity.respond_to? :raw
    json.published activity.time_created.utc.iso8601
    json.actor do
      creator = activity.created_by
      creator = activity.source.class.to_s.underscore.humanize if creator.blank?
      json.id "tag:#{request.host},#{Time.now.year}:#{creator}"
    end

    json.verb case activity
              when Build
                'build'
              when Commit
                'add'
              when ReviewRequest
                if activity.status == 'pending'
                  'request'
                elsif activity.status == 'discarded'
                  'retract'
                elsif activity.status == 'submitted'
                  'complete'
                else
                  'post'
                end
              else
                'post'
              end

    json.object do
      dash_type = activity._type.snakecase.dasherize
      json.url activity.respond_to?(:target_url) ? activity.target_url : polymorphic_url(activity, project: current_project)
      json.id "tag:#{request.host},#{Time.now.year}:#{activity.opaque_id}"

      json.object_type case activity
                       when Build
                         'build'
                       when Commit
                         'commit'
                       when ReviewRequest
                         'review'
                       else
                         dash_type
                       end
    end

    json.target do
      json.url project_url(current_project)
    end
  end
end
