#
# Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#
require File.expand_path('../boot', __FILE__)

require File.expand_path('../../lib/overridable_config', __FILE__)

# Pick the frameworks you want:
require "action_controller/railtie"
require "action_mailer/railtie"
require "sprockets/railtie"

if defined?(Bundler)
  # Require the gems listed in Gemfile, including any gems
  # you've limited to :test, :development, or :production.
  Bundler.require(:default, Rails.env)
end

module Orchestrate
  class Application < Rails::Application

    # Sync stdout output so logging for Foreman works.
    $stdout.sync = true

    # Settings in config/environments/* take precedence over those specified here.
    # Application configuration should go into files in config/initializers
    # -- all .rb files in that directory are automatically loaded.

    # Custom directories with classes and modules you want to be autoloadable.
    config.autoload_paths += Dir["#{config.root}/lib/**/","#{Rails.root}/app/services/**/" ]
    # find services and models (from services)
    Dir.glob("#{Rails.root}/app/services/**/*.rb").each {|f| require f}


    # Fix paths for tests
    $: << Rails.root

    # Only load the plugins named here, in the order given (default is alphabetical).
    # :all can be used as a placeholder for all plugins not explicitly named.
    # config.plugins = [ :exception_notification, :ssl_requirement, :all ]

    # Activate observers that should always be running.
    # config.active_record.observers = :cacher, :garbage_collector, :forum_observer

    # Set Time.zone default to the specified zone and make Active Record auto-convert to this zone.
    # Run "rake -D time" for a list of tasks for finding time zone names. Default is UTC.
    # config.time_zone = 'Central Time (US & Canada)'

    # Raise an exception if a locale is not available
    I18n.config.enforce_available_locales = true

    # The default locale is :en and all translations from config/locales/*.rb,yml are auto loaded.
    # config.i18n.load_path += Dir[Rails.root.join('my', 'locales', '*.{rb,yml}').to_s]
    # config.i18n.default_locale = :de

    # Configure the default encoding used in templates for Ruby 1.9.
    config.encoding = "utf-8"

    # Configure sensitive parameters which will be filtered from the log file.
    config.filter_parameters += [:password, :password_confirmation]

    # Enable the asset pipeline
    config.assets.enabled = true

    # Version of your assets, change this if you want to expire all your assets
    config.assets.version = '1.0'

    # Set Redis as the cache store
    config.cache_store = :redis_store, 'redis://localhost:6379/0/cache', { expires_in: 90.minutes }

    # serve HTML Assets (for HTTP error pages)
    config.assets.paths << Rails.root.join('app/assets/html')
    config.assets.register_mime_type('text/html', '.html')

    config.middleware.insert_after ActionDispatch::ParamsParser, ActionDispatch::XmlParamsParser

    config.generators do |g|
      g.orm :mongo_mapper
      g.template_engine :erb
      g.test_framework :rspec, :fixture => false
      g.javascripts false
      g.javascript_engine :js
      g.stylesheet_engine :scss
    end

    responses = { 'MongoMapper::DocumentNotFound' => :not_found }

    if rescue_responses = config.action_dispatch.rescue_responses
      rescue_responses.update(responses)
    else
      ActionDispatch::ShowExceptions.rescue_responses.update(responses)
    end

    config.local_settings = OverridableConfig.new
    config.teamforge = config.local_settings.load('teamforge').with_indifferent_access
    config.security = config.local_settings.load('security').with_indifferent_access
    config.rabbitmq = config.local_settings.load('rabbitmq').with_indifferent_access

    # Needed for devise confirmation emails
    config.action_mailer.default_url_options = { :host => "yourdomain.com" }

    # Directory that store TeamForge WSDLs
    config.wsdl_directory = Rails.root.join('config/wsdl').to_s

    revision_file = Rails.root.join('config', 'REVISION')
    begin
      if revision_file.exist?
        config.revision = revision_file.open { |f| f.read }.chomp
      elsif Rails.root.join('.git').exist?
        config.revision = "Development at revision: #{`git log -1 --pretty=format:"%h"`.chomp}"
      end
    rescue => e
      config.revision = "Unknown development version"
    end

    # when Orchestrate the web app is started we want to synchronize the tracker sources (world state) with CTF. Will be triggered
    # every time you restart an nginx service.
    if Rails.env.production?
      config.after_initialize do
        Resque.enqueue(WorkItemIntegrationUtils::Upgrader) if TeamForge::ProjectConfiguration.where(tracker_source_id: nil).count > 0
        TeamForge::ProjectConfiguration.all.each do |config|
          Resque.enqueue(OrchestrateSyncListenerService, config.ctf_project_id)
        end
      end
    end
  end
end

module Mongo

  # @deprecated Use Mongo::MongoReplicaSetClient instead. Support will be removed after v2.0
  # Please see old documentation for the ReplSetConnection class
  class ReplSetConnection < MongoReplicaSetClient
    include Mongo::LegacyWriteConcern

    def initialize(*args)
      if args.last.is_a?(Hash)
        opts = args.pop
        write_concern_from_legacy(opts)
        # PATCHED BELOW TO PROPERLY SEND HOSTS
        args = [args].push(opts)
        # END PATCH
      end
      super
    end
  end
end
