Orchestrate::Application.configure do
  # Settings specified here will take precedence over those in config/application.rb

  # Skip enforcing available locales, silently defaults to config.i18n.default_locale (:en)
  I18n.config.enforce_available_locales = false

  # Code is not reloaded between requests
  config.cache_classes = true
  config.eager_load = true

  # Full error reports are disabled and caching is turned on
  config.consider_all_requests_local       = false
  config.action_controller.perform_caching = true

  # Disable Rails's static asset server (Apache or nginx will already do this)
  #config.serve_static_assets = false
  config.serve_static_assets = true

  # Compress JavaScripts and CSS
  config.assets.compress = false

  # Don't fallback to assets pipeline if a precompiled asset is missed
  config.assets.compile = false

  # Generate digests for assets URLs
  config.assets.digest = true

  # Precompile additional assets (application.js, application.css.scss, and all non-JS/CSS are already added)
  config.assets.precompile += %w( *.js *.css )

  # Precompile HTML assets (used for http error pages)
  config.assets.precompile += %w(*.html)

  #JS compressor, includes copyright
  config.assets.js_compressor = :uglifier

  # Defaults to Rails.root.join("public/assets")
  # config.assets.manifest = YOUR_PATH

  # Specifies the header that your server uses for sending files
  # config.action_dispatch.x_sendfile_header = "X-Sendfile" # for apache
  # config.action_dispatch.x_sendfile_header = 'X-Accel-Redirect' # for nginx

  # Force all access to the app over SSL, use Strict-Transport-Security, and use secure cookies.
  # config.force_ssl = true

  # See everything in the log (default is :info)
  # config.log_level = :info

  # Use a different logger for distributed setups
  # config.logger = SyslogLogger.new
  config.logger = ActiveSupport::Logger.new(File.new('/var/log/eventq/server.log', 'a'))

  # Set log level for our custom logger
  config.logger.level = Logger::INFO

  # Use a different cache store in production
  config.cache_store = :redis_store

  # Enable serving of images, stylesheets, and JavaScripts from an asset server
  # config.action_controller.asset_host = "http://assets.example.com"

  # Disable delivery errors, bad email addresses will be ignored
  # config.action_mailer.raise_delivery_errors = false

  # Enable threaded mode
  # config.threadsafe!

  # Enable locale fallbacks for I18n (makes lookups for any locale fall back to
  # the I18n.default_locale when a translation can not be found)
  config.i18n.fallbacks = true

  # Send deprecation notices to registered listeners
  config.active_support.deprecation = :notify

  # Halt startup if teamforge mode is disabled via user configuration
  require 'config/teamforge_config_verification.rb'



  #################################################################################
  # Maintains environment constants for the New System
  #
  # BIGLOCK:
  # states : 2 - use the new interpreter system only (obsolete the legacy system)
  #          1 : maintain the legacy and the new system simultaneously
  #          0 : maintain the legacy system only
  # Note: BIGLOCK determines if a data packet is to be sent to the legacy
  # system
  #
  # SCHEDULER_QUEUE_SIZE: maximum queue size  for data packets awaiting
  # processing valid values 0 ...5000
  # If the queue size is set to zero all packets not immediately processed
  # will be discarded
  #
  # SCHEDULER_SLEEP - scheduler sleeps for this many seconds and then wakes
  # up
  #
  # PERFECT_LEGACY_EMULATION: The views will contain ditto copies of
  # documents persisited in the legacy system
  #
  # RABBITMQ_SWITCH: push data packets to the legacy system through Rabbit
  # or do direct insertions in the legacy system (OFF value does not work properly
  # right now)
  #
  # NAMESPACE_SEPARATOR: substitute for dots in key names with this separator
  #
  # SERVICE_LOG: write to the service log
  #
  # SHOW_NAMESPACE_TRAVERSAL: show the traversal through a nested hash - used
  # for debugging
  #
  # STACK_TRACE: invoke the stack trace on the given service
  #
  ################################################################################


    #----------------------------------------------------------------------
    # set the BIGLOCK VALUE
    # BIGLOCK == 0 implies only persist packets in the legacy system
    # BIGLOCK == 1 implies persist packets into the new and the legacy system
    # BIGLOCK == 2 implies only persist packets in the new system system
    #------------------------------------------------------------------------

    config.BIGLOCK = 0


    #-----------------------------------------------------------
    # scheduler queue size: Maximum number of packets permitted
    # in the queue for future processing
    #------------------------------------------------------------

  config.SCHEDULER_QUEUE_SIZE = 1000


    #------------------------------------------------------------
    # Scheduler Sleep: sleep for so many (float) seconds.
    # must have a value >= 1 and less than 60
    #------------------------------------------------------------

  config.SCHEDULER_SLEEP = 5

    #-----------------------------------------------------------------------
    # use RabbitMQ to insert documents into the legacy activity collection
    # if false the system will use direct insertion into the legacy system
    #-----------------------------------------------------------------------

  config.RABBITMQ_SWITCH = true


    #-----------------------------------------------------------------------
    # If set then a packet will be inserted into the new system only iff
    # (mathematical iff) it is inserted into the legacy system
    #-----------------------------------------------------------------------

  config.PERFECT_LEGACY_EMULATION  = true


    #-----------------------------------------------------------------------
    # NAMESPACE_SEPARATOR: mongoDB does not permit a document key to have
    # dots in the name. We namespace packet keys with dot separators as
    # (for example): fields.status.name
    # The namespace separator is used to substitute dots in a namespaced
    # key (i.e attribute) prior to insertion of a packet into a view or
    # the legacy system
    #-----------------------------------------------------------------------

  config.NAMESPACE_SEPARATOR = '_'


    #-----------------------------------------------
    # Write To  Service Log
    #-----------------------------------------------

  config.SERVICE_LOG = false

  config.SHOW_NAMESPACE_TRAVERSAL = false


    #----------------------------------------------
    # maintain a stack trace for the services
    #----------------------------------------------

  config.STACK_TRACE_COUPLER             = false
  config.STACK_TRACE_DATADUMP            = false
  config.STACK_TRACE_EMITTER             = false
  config.STACK_TRACE_EMULATOR            = false
  config.STACK_TRACE_EXECUTOR            = false
  config.STACK_TRACE_INTERPRETER         = false
  config.STACK_TRACE_PATTERN             = false
  config.STACK_TRACE_QUEUE               = false
  config.STACK_TRACE_RABBIT_CONNECTOR    = false
  config.STACK_TRACE_PACKET_INJECTOR     = false
  config.STACK_TRACE_SCHEDULER           = false
  config.STACK_TRACE_TOKENIZER           = false
  config.STACK_TRACE_PACKET_MAP          = false


end
