# initializer added so that EventMachine/AMQP work with Phusion Passenger's
# forking

# EventMachine needs to run in it's own thread... sometimes. If you're using Thin for a web server (which we use in development),
# it has EventMachine built in. But if you you're using Passenger (which we use in production), its way of forking processes can
# cause orphaned threads. Finally, if you're using Capybara for testing (which we are), then there's third way to make it work. -
# See more at: http://www.hiringthing.com/2011/11/04/eventmachine-with-rails.html#sthash.yrvyYXiM.dpuf

# Solution from http://lcd.blogs.southwales.ac.uk/2013/01/21/rabbit-mq-ampq-publishing-messages-to-the-q/


module AdmitEventMachine
  def self.start
    if defined?(PhusionPassenger)
      PhusionPassenger.on_event(:starting_worker_process) do |forked|
        if forked && EM.reactor_running?
          EM.stop
        end
        Thread.new {
          EM.run
        }
        die_gracefully_on_signal
      end
    end
  end

  def self.die_gracefully_on_signal
    Signal.trap("INT") { EM.stop }
    Signal.trap("TERM") { EM.stop }
  end
end

AdmitEventMachine.start