#
# Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#
require 'migrations/exceptions'
class CopyDataFromLatestActivityToSummary < Migration

  extend MigrationHelper


  DOCUMENT_BLOCK_SIZE = 100
  ASCENDING = 1
  DESCENDING = -1


  FIELDS_TO_COPY = {'CustomSummary' => ['status_name', 'status_type', 'created_by', 'summary', 'description', 'link_to', 'custom_data', 'associated_remote_id'],
                    'WorkItemSummary' => ['summary', 'description', 'closed', 'deleted', 'priority', 'assigned_to', 'creation_time', 'tags']}
  def self.up
    ['WorkItemSummary', 'CustomSummary'].each do |activity_type|
      self.copy_latest_data(activity_type)
    end
  end

  def self.copy_latest_data(activity_type)
    (0..document_count(:activity_summaries, _type: activity_type)).step(DOCUMENT_BLOCK_SIZE) do |records_to_skip|
      iterate_over(:activity_summaries, {_type: activity_type}, {timeout: false, sort: [['_id', ASCENDING]], limit: DOCUMENT_BLOCK_SIZE, skip: records_to_skip}) do |activity_summary|
        latest_activity = within(:activities) { |activities| activities.find_one({activity_summary_id: activity_summary['_id']},
                                                                                         {sort: [['time_created', DESCENDING]], limit: 1}) }
        data_to_insert = latest_activity.slice(*FIELDS_TO_COPY[activity_type])
        db[:activity_summaries].update({'_id' => activity_summary['_id']}, {'$set' => data_to_insert})
      end
    end
  end

  def self.down
    ['WorkItemSummary', 'CustomSummary'].each do |activity_type|
      self.remove_copied_data(activity_type)
    end
  end

  def self.remove_copied_data(activity_type)
    (0..document_count(:activity_summaries, _type: activity_type)).step(DOCUMENT_BLOCK_SIZE) do |records_to_skip|
      iterate_over(:activity_summaries, {_type: activity_type}, {timeout: false, sort: [['_id', ASCENDING]], limit: DOCUMENT_BLOCK_SIZE, skip: records_to_skip}) do |activity_summary|
        data_to_update = array_to_hash(FIELDS_TO_COPY[activity_type]){ |field| nil}
        db[:activity_summaries].update({'_id' => activity_summary['_id']}, {'$set' => data_to_update})
      end
    end
  end
end