#
# Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#
require 'migrations/exceptions'
class FlattenNestedStructuresInActivityAndSummaryClasses < Migration

  extend MigrationHelper
  FIELDS_TO_RENAME = {
      build: {
          up: {'status.type' => 'status_type', 'status.name' => 'status_name',
               'test_results.passed_count' => 'test_results_passed_count', 'test_results.failed_count' => 'test_results_failed_count',
               'test_results.ignored_count' => 'test_results_ignored_count', 'test_results.url' => 'test_results_url'
          },
          down: {'status_type' => 'status.type', 'status_name' => 'status.name',
               'test_results_passed_count' => 'test_results.passed_count', 'test_results_failed_count' => 'test_results.failed_count',
               'test_results_ignored_count' => 'test_results.ignored_count', 'test_results_url' => 'test_results.url'
          }
      },
      review: {
          up: {'status.type' => 'status_type', 'status.name' => 'status_name'},
          down: {'status_type' => 'status.type', 'status_name' => 'status.name'}
      }
  }

  FIELDS_TO_UNSET = {
      build: {
          up: {'status' => '', 'test_results' => ''},
          down: {'status_name' => '', 'status_type' => '', 'test_results_passed_count' => '', 'test_results_failed_count' => "",
                 'test_results_ignored_count' => '', 'test_results_url' => ''
          }
      },
      review: {
          up: {'status' => ''},
          down: {'status_name' => '', 'status_type' => ''}
      }

  }


  def self.up
    #review requests
    db[:activities].update({:_type =>  'ReviewRequest'}, {'$rename' => FIELDS_TO_RENAME[:review][:up]}, multi: true)
    db[:activities].update({:_type =>  'ReviewRequest'}, {'$unset' => FIELDS_TO_UNSET[:review][:up]}, multi: true)

    #builds
    db[:activities].update({:_type => 'Build'}, {'$rename' => FIELDS_TO_RENAME[:build][:up]}, multi: true)
    db[:activities].update({:_type =>'Build'}, {'$unset' => FIELDS_TO_UNSET[:build][:up]}, multi: true)

    #review request summaries
    db[:activity_summaries].update({:_type =>  'ReviewRequestSummary'}, {'$rename' => FIELDS_TO_RENAME[:review][:up]}, multi: true)
    db[:activity_summaries].update({:_type =>  'ReviewRequestSummary'}, {'$unset' => FIELDS_TO_UNSET[:review][:up]}, multi: true)

    #build summaries
    db[:activity_summaries].update({:_type => 'BuildSummary'}, {'$rename' => FIELDS_TO_RENAME[:build][:up]}, multi: true)
    db[:activity_summaries].update({:_type =>'BuildSummary'}, {'$unset' => FIELDS_TO_UNSET[:build][:up]}, multi: true)
  end

  def self.down
    #review requests
    db[:activities].update({:_type =>  'ReviewRequest'}, {'$rename' => FIELDS_TO_RENAME[:review][:down]}, multi: true)
    db[:activities].update({:_type =>  'ReviewRequest'}, {'$unset' => FIELDS_TO_UNSET[:review][:down]}, multi: true)

    #builds
    db[:activities].update({:_type => 'Build'}, {'$rename' => FIELDS_TO_RENAME[:build][:down]}, multi: true)
    db[:activities].update({:_type =>'Build'}, {'$unset' => FIELDS_TO_UNSET[:build][:down]}, multi: true)

    #review request summaries
    db[:activity_summaries].update({:_type =>  'ReviewRequestSummary'}, {'$rename' => FIELDS_TO_RENAME[:review][:down]}, multi: true)
    db[:activity_summaries].update({:_type =>  'ReviewRequestSummary'}, {'$unset' => FIELDS_TO_UNSET[:review][:down]}, multi: true)

    #build summaries
    db[:activity_summaries].update({:_type => 'BuildSummary'}, {'$rename' => FIELDS_TO_RENAME[:build][:down]}, multi: true)
    db[:activity_summaries].update({:_type =>'BuildSummary'}, {'$unset' => FIELDS_TO_UNSET[:build][:down]}, multi: true)
  end
end
