#
# Copyright 2015 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#
require 'migrations/exceptions'
class RemovePipelineAndAssociateBuildCustomAndReviewSourcesToProject < Migration

  extend MigrationHelper

  STEP_TYPE_CHECK = %w(build custom review_request)

  # Method for rake db:migrate command
  def self.up
    self.link_sources_to_project
    self.remove_external_activities
    # Intentionally commented. ToDo: To be uncommented or moved to work-item / commit sources migration.
    # self.remove_pipelines_collection
  end

  # Method to migrate Review, Build and Custom sources to be associated to the project.
  # For each pipeline in the project, get each of the steps in the pipeline. Save the project that the pipeline belongs to.
  # For the sources in each step, update the Project_id field.
  def self.link_sources_to_project
    iterate_over(:pipelines) do |pipeline|
      this_project = pipeline['project_id']
      steps = pipeline['pipeline_steps']

      steps.each { |each_step|
        if STEP_TYPE_CHECK.include? each_step['type']
          source_ids = each_step['source_ids']
          source_ids.each { |each_source_id|
            db[:activity_sources].update({'_id' => each_source_id}, {'$set' => {'project_id' => this_project}})
          }
        end
      }

    end
  end

  def self.remove_external_activities
    db[:activities].remove({_type: "External"})
  end

  def self.remove_pipelines_collection
    db[:pipelines].drop()
  end

end