#
# Copyright 2015 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#
require 'migrations/exceptions'
require 'migrations/remove_pipelines/within_project_scenario'
require 'migrations/remove_pipelines/outside_project_scenario'

class RemovePipelinesAndAssociateCommitSourcesToProject < Migration

  extend MigrationHelper

  def self.up
    new_sources = []; old_sources = [];
    # 1. pipelines grouped by Project
    pipelines_for_project_bson = db[:pipelines].aggregate( [ { '$group' => { '_id' => "$project_id", 'pipe_ids' => { '$push' => "$_id" } } } ] )
    pipelines_for_project = {}
    pipelines_for_project_bson.map{|hash| pipelines_for_project[hash["_id"]] = hash["pipe_ids"]}

    # 2. Get a map of sources to project
    source_to_project = {}
    pipelines_for_project.each do |project, pipelines|
      pipelines.each do |pipeline_id|
        steps = WithinProjectScenario.get_source_steps_for_pipeline(pipeline_id, 'commit')
        steps.each do |each_step|
          source_ids = each_step['source_ids']
          source_ids.each do |source_id|
            source_to_project[source_id] = project
          end
        end
      end
    end

    # create temp collection to hold source_project_repo_uri info
    # so we can aggregate the data again grouped by project_id and repository_uri
    self.create_temp_source_collection(source_to_project)
    sources_grouped_by_proj_and_repo_uri_bson = db[:source_project_repo_uri].aggregate( [ { '$group' => { '_id' => {"proj" => "$project_id",
                                                                                                                    "repository_uri" => "$repository_uri"},
                                                                                                          'source_ids' => {'$push' => "$original_id"} }} ])

    sources_grouped_by_proj_and_repo_uri_bson.each do | data |
      status = nil
      data['source_ids'].uniq!
      # duplicate commit sources
      if data['source_ids'].count > 1
        # get common status from duplicate source
        status = self.get_status_sources(data['source_ids'])
      end

      unless opts = self.find_temp_source_by_status_in_sources(data['source_ids'], status)
        puts "Skip, Source not found : data['source_ids']: #{data['source_ids'].inspect} with status: #{status.inspect}"
        next
      end

      # create new commit source
      new_source = self.create_new_commit_source(opts)
      new_sources << new_source

      if data['source_ids'].count > 1
        # merge duplicate associations
        WithinProjectScenario.merge_association_summaries_for_source(data['source_ids'], new_source, 'commit')
      else
        # Reconnect activities & activity summaries
        OutsideProjectScenario.reconnect_activity_summaries_to_new_source(data['source_ids'].first, new_source)
        OutsideProjectScenario.reconnect_activities_to_new_source(data['source_ids'].first, new_source)
      end

      old_sources += data['source_ids']
    end

    # mark to delete for old commit sources
    OutsideProjectScenario.mark_to_delete(old_sources)

    # drop the temporarily created collection
    db[:source_project_repo_uri].drop()

  end

  # create a temporary collection to hold source & project & repository_uri information
  def self.create_temp_source_collection(source_to_project)
    db.create_collection("source_project_repo_uri")
    source_to_project.each do |source, project|
      next if !source.present? || !project.present?
      if source_data = WithinProjectScenario.find_source_by_id(source)
        if repo_uri = source_data['repository_uri']
          db[:source_project_repo_uri].insert(source_data.except('_id').merge!("project_id" => project).merge!('original_id' => source))
        end
      end
    end
  end

  # create a new commit source
  def self.create_new_commit_source(data)
    new_data = db[:activity_sources].insert(data.except('_id').except('original_id'))
    new_data
  end

  # get the temp source data in the temp collection
  # if we don't set the status, we will search all sources without status checking
  def self.find_temp_source_by_status_in_sources(original_ids, status=nil)
    source = nil
    new_query = status.nil? ? { 'original_id' => { '$in' => original_ids} } : { 'original_id' => { '$in' => original_ids}, 'active' => status }

    db[:source_project_repo_uri].find(new_query) do |cursor|
      source = cursor.first
    end

    source
  end

  # get the status for sources
  # if all sources are same status, we will use that status
  # if at least one source is active, we will use active status
  def self.get_status_sources(source_ids)
    ActivitySource.where(id: source_ids).map(&:active).include?(true) ? true : false
  end

end