#
# Copyright 2015 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#
require 'migrations/exceptions'
class AddCreatedByFieldToBuildSummaries < Migration

  extend MigrationHelper

  def self.up

    # Create a compound index that lists activity summaries in sequential manner
    custom_index = db[:activities].create_index({'_type' => Mongo::ASCENDING, 'activity_summary_id' => Mongo::ASCENDING, 'time_created' => Mongo::DESCENDING})
    # Extract builds using the index.
    activities = db[:activities].find({'_type' => 'Build'}, opts = {named_hint: custom_index, fields: {_id: 0, activity_summary_id: 1, created_by: 1, time_created: 1}}).to_a

    # Hash of build_summary_id to created_by
    summary_to_created_by = {}

    # Convert the array of build activity hashes to a hash
    activities.each do |activity|
      summary_to_created_by[activity['activity_summary_id']] = activity['created_by']
    end

    # Update build summary objects
    summary_to_created_by.each do |summary_key, created_by|
      db[:activity_summaries].update({'_id' => summary_key }, {'$set' => {'created_by' => created_by} })
    end

    # Drop compound index
    db[:activities].drop_index(custom_index)
  end


  # Rollback the action of adding created_by to the BuildSummary objects
  # Flow us similar to the up method. To remove the created_by field, $unset param is used instead of $set
  def self.down

    custom_index = db[:activities].create_index({'_type' => Mongo::ASCENDING, 'activity_summary_id' => Mongo::ASCENDING, 'time_created' => Mongo::DESCENDING})
    activities = db[:activities].find({'_type' => 'Build'}, opts = {named_hint: custom_index, fields: {_id: 0, activity_summary_id: 1, created_by: 1, time_created: 1}}).to_a

    activities.each do |activity|
      db[:activity_summaries].update({'_id' => activity['activity_summary_id'] }, {'$unset' => {'created_by' => ''} })
    end

    db[:activities].drop_index(custom_index)
  end
end
