###################################################################################
# Algorithm: Each TrackerSource is first marked for deletion with the to_delete flag.
# Then, each activity source is examined. If the to_delete attribute  is set
# and if the active attribute is also set then the latter attribute is mutated
# to false
# Performs verification by outputting the to_delete and activity attribute for
# all activity sources after record mutations are performed.
#
# Testing The migration on the development database
#  [1] run: deactivate_sources_driver.rb development from the command line
#      This randomly mutates the to_delete and activity attributes
#  [2] Run the migration and examine the output
#
#  Test process (driver.rb must be available):
#
#  ruby driver.rb
#  rake db:rollback
#  rake db: migrate
#   - and repeat this cycle as require
#
# Note: (1) Not all activity sources will contain a to-delete attribute
#       (2) There is no down migration
#
###################################################################################


require 'migrations/exceptions'


###################################################################################
# Migration Class#
###################################################################################

class ActivitySourceFixToDeleteLogic < Migration

  extend MigrationHelper

  def self.up

    tracker_sources_updated = 0
    tracker_sources_update_failure = 0
    update_failures = []

    # Update tracker sources and mark them for deletion.
    iterate_over(:activity_sources, {'_type' => 'TrackerSource'}) do |a_tracker_source|
      update_status = db[:activity_sources].update({'_id' => a_tracker_source['_id']}, {'$set' => {'to_delete' => true, 'project_id' => nil}})

      if update_status['updatedExisting'] && (update_status['err'] == nil)
        tracker_sources_updated += 1
      else
        tracker_sources_update_failure += 1
        update_failures << "ID: #{a_tracker_source['_id']}, Name: #{a_tracker_source['display_name']}"
      end
    end

    puts "#{tracker_sources_updated} Tracker Sources marked for deletion and disassociated from the project."
    unless tracker_sources_update_failure == 0
      puts "Update failed for the following #{tracker_sources_update_failure} Tracker Sources:"
      update_failures.each { |failure| puts failure}
    end

    # Deactivate all sources marked for deletion.
    db['activity_sources'].find({ to_delete:  { '$exists' => true }} ).to_a.each do |record|
      if record['to_delete'] && record['active']
        db[:activity_sources].update({'_id' => record['_id']}, {'$set' => {'active' => false} })
      end
    end

    #################################
    # activity source verification
    #################################

    db['activity_sources'].find({ to_delete:  { '$exists' => true }} ).to_a.each do |record|

      if record['to_delete'] && record['active']
        raise "migration error - invalid activity source: " + record.inspect
      end
    end
  end
  
end
