#
# Copyright 2015 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#
require 'migrations/exceptions'
class AddActiveToProject < Migration

  extend MigrationHelper

  def self.up
    add_fields(:projects, 'active' => true)

    # Foreign key for project in the project_config record presumed to be project_url.
    # Or could be name?  See extract_ctf_session - it seems to favor url over name.

    iterate_over(:projects) do |proj|
      configs = db['team_forge.project_configurations'].find({'project_url' => proj['name']}, sort: ['ctf_project_id', :descending]).to_a
      config = nil

      configs.each do |current_config|
        if current_config['project_id'] == proj['_id']
          config = current_config
          break
        end
      end

      if config
        if configs.count > 1
          puts "WARNING: Found more than one configuration for '#{proj['name']}'; using #{config['ctf_project_id']}"
        end
        puts "Setting project '#{proj['name']}' active flag to #{config['active']}"
        db[:projects].update({'_id' => proj['_id']}, {'$set' => {'active' => config['active']}})
      else
        puts "WARNING: No project configuration found for #{proj['name']}"
      end
    end
  end

  def self.down
    # remove fields; all we can do.
    remove_fields(:projects, ['active'], true)
  end
end
