#
# Copyright 2015 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#
require 'migrations/exceptions'
class ReconnectCommitAndBuildReference < Migration

  extend MigrationHelper

  def self.up
    commit_source_ids = CommitServer.where(:to_delete.ne => nil, :project_id => nil).all.map(&:_id)
    builds = BuildServer.where(:commit_server_id => commit_source_ids).all

    reconnect_summaries = []
    commit_summaries = []
    old_new_source = []
    builds.each do | build |
      # get the new commit server for the build that has old commit server reference by repository_uri
      if new_cs = CommitServer.where(:repository_uri => build.commit_server.repository_uri, :to_delete.ne => true).first
        old_cs = build.commit_server

        # get the ActivitySummary by old Commit Source
        commit_summaries = ActivitySummary.where(:source_id => old_cs.id, :settled => false, :message => nil).all

        commit_summaries.each do | summary |
          # get the commit information so we can attach to commit summary
          if commit = Commit.where(:revision_number => summary.revision_number, :source_id => [old_cs.id, new_cs.id]).first
            commit.update_attribute(:source_id, new_cs.id)
            summary.populate_fields(commit)
            activity_summaries_from_message = CommitSummary.associations_from_commit_message(commit)
            summary.activity_summaries << activity_summaries_from_message
            summary.settled = true
            summary.source_id = new_cs.id
            summary.save

            reconnect_summaries << summary
          end
        end
        old_new_source << {old_cs.id => new_cs.id}

        # re-associate the new commit server to build server
        build.update_attribute(:commit_server_id, new_cs.id)
      end
    end
    # Display the relation with old and new commit source
    puts "Commit Sources Relation (old => new)\n"
    puts "#{old_new_source}\n\n"

    # Display the original commit summaries
    puts "Original Commit Summaries \n"
    puts "#{commit_summaries.map(&:id)}\n\n"

    # Display the new reconnected commit summaries
    puts "New Commit Summaries \n"
    puts "#{reconnect_summaries.map(&:id)}\n\n"
  end
end