#
# Copyright 2016 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#
require 'migrations/exceptions'
class WorkItemAssociationFix < Migration

  extend MigrationHelper

  # 1. find out phantom work item activity summaries.
  # 2. Delete #1 list
  # 3. find out duplicate activity summaries.
  # 4. find out the latest activity summary from the #3.
  # 5. Links all activities which link with old activity summary to new activity summary.
  # 6. Delete old activity summaries.
  def self.up
    # Step 1 and 2
    # find and delete phantom work item activity summaries.
    ActivitySummary.delete_all(:_type => 'WorkItemSummary', :settled => false)

    # Step 3
    # Find out duplicate activity summaries.
    duplicate_wis_remote_ids =  db[:activity_summaries].aggregate([
                                                                       {'$group' => { '_id' => "$remote_id",
                                                                                      'count' => { '$sum' => 1 }, '_type' => { '$push' => '$_type' }}},
                                                                       {'$match' => { 'count' => { '$gt' => 1 }, '_type' => 'WorkItemSummary' }}
                                                                   ])
    old_activities = []
    # Step 4
    # Find out the latest activity summary from the #3.
    duplicate_wis_remote_ids.map do | dup_remote_id |
      unless dup_remote_id['_id'].nil?
        new_associated_activity_summary_ids = []
        dup_wis = ActivitySummary.where('remote_id' => dup_remote_id['_id'], '_type' => 'WorkItemSummary').sort({'updated_at' => -1}).all

        # Get the latest work item summary
        # and now dup_wis contains only old work item summary
        new_wis = dup_wis.shift

        # Step 5
        # Links all activities which link with old activity summary to new activity summary.
        if new_work_item_source = new_wis.source
          # re-link the all activity associations
          dup_wis.map do | old_wis |
            new_associated_activity_summary_ids += old_wis.associated_activity_summary_ids.to_a
            old_wis.activities.map do | activity |
              old_activities << { activity.id => { activity.activity_summary_id => activity.source_id} }
              if status_id = self.corresponding_tracker_status(new_wis, activity)
                activity.remote_status_id = status_id
                puts "This activity's status does not have Source/Tracker : #{activity.inspect}"
              end
              activity.source_id = new_work_item_source.id
              activity.activity_summary_id = new_wis._id
              activity.save(validate: false)
            end
          end
        end
        # Re-link the all summary associations
        new_associated_activity_summary_ids += new_wis.associated_activity_summary_ids.to_a
        new_wis.update_attribute(:associated_activity_summary_ids, new_associated_activity_summary_ids.uniq) if new_associated_activity_summary_ids

        # This is a debug log to verify changes.
        puts "Before Activities change: {:activity.id, :activity.activity_summary_id, :activity.source_id} \n #{old_activities.join("\n")}"

        # Step 6
        # Delete old work item summaries
        ActivitySummary.delete_all(:_type => 'WorkItemSummary', :_id => dup_wis.map(&:_id))
      end
    end
  end

  # Find and return tracker status id based new activity summary with old work item
  def self.corresponding_tracker_status(new_wis, work_item)
    new_tracker_status = {}; old_tracker_status = {}
    new_wis_tracker = new_wis.source.tracker
    old_wis_tracker = work_item.source.tracker
    if new_wis_tracker && old_wis_tracker
      new_wis_tracker.statuses.map { |s| new_tracker_status[s.name] = s.remote_id }
      old_wis_tracker.statuses.map { |s| old_tracker_status[s.remote_id] = s.name }
      new_tracker_status[old_tracker_status[work_item.remote_status_id]]
    else
      nil
    end
  end
end