#
# Copyright 2016 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#
require 'migrations/exceptions'
class GeneratePredefinedWebHookData < Migration

  extend MigrationHelper

  def self.up
    pattern1 = {
        'pattern_id' => 'jira1',
        'pattern_name' => 'jira_cloud',
        'view_name' => 'jiraView',
        'required_attributes' => 'webhookEvent,issue.key,issue.fields.description,issue.fields.summary,issue.fields.created,issue.fields.priority.id,issue.fields.status.id,issue.fields.assignee.name,issue.fields.project.key,issue.fields.issuetype.name,issue.fields.status.name,issue.fields.project.id,issue.fields.issuetype.iconUrl,issue.fields.project.self,issue.fields.status.statusCategory.key,issue.fields.created,issue.fields.creator.name,user.name',
        'append_always' => 'true',
        'projection' =>  'source_association_key,webhookEvent,issue.key,issue.fields.description,issue.fields.summary,issue.fields.created,issue.fields.priority.id,issue.fields.status.id,issue.fields.assignee.name,issue.fields.project.key,issue.fields.issuetype.name,issue.fields.status.name,issue.fields.project.id,issue.fields.issuetype.iconUrl,issue.fields.project.self,issue.fields.status.statusCategory.key,issue.fields.created,issue.fields.creator.name,user.name'
    }

    pattern2 = {
        'pattern_id' => 'jira2',
        'pattern_name' => 'jira_cloud',
        'view_name' => 'jiraView',
        'required_attributes' => 'webhookEvent,issue.key,issue.fields.description,issue.fields.summary,issue.fields.created,issue.fields.priority.id,issue.fields.status.id,issue.fields.assignee,issue.fields.project.key,issue.fields.issuetype.name,issue.fields.status.name,issue.fields.project.id,issue.fields.issuetype.iconUrl,issue.fields.project.self,issue.fields.status.statusCategory.key,issue.fields.created,issue.fields.creator.name,user.name',
        'append_always' => 'true',
        'projection' => 'source_association_key,webhookEvent,issue.key,issue.fields.description,issue.fields.summary,issue.fields.created,issue.fields.priority.id,issue.fields.status.id,issue.fields.assignee,issue.fields.project.key,issue.fields.issuetype.name,issue.fields.status.name,issue.fields.project.id,issue.fields.issuetype.iconUrl,issue.fields.project.self,issue.fields.status.statusCategory.key,issue.fields.created,issue.fields.creator.name,user.name'
    }

    packet_map1 = {
        'packet_type' => 'jira_cloud',
        'group_id' => '1',
        'jira_cloud' =>  {
            'id' => 'issue.key',
            'summary' => 'issue.fields.summary',
            'description' => 'issue.fields.description',
            'status_id' => 'issue.fields.status.id',
            'status_name' => 'issue.fields.status.name',
            'status_category' => 'issue.fields.status.statusCategory.key',
            'assigned_to' => 'issue.fields.assignee.name',
            'priority' => 'issue.fields.priority.id',
            'tracker_url' => 'issue.fields.issuetype.iconUrl',
            'tracker_key' => 'issue.fields.project.key',
            'tracker_name' => 'issue.fields.issuetype.name',
            'remote_id' => 'issue.fields.project.id',
            'creation_time' => 'issue.fields.created',
            'created_by' => 'issue.fields.creator.name',
            'updated_by' => 'user.name',
            'go_url' => 'issue.fields.project.self',
            'event_name' => 'webhookEvent'
        },
        'description' => 'basic jira cloud mapper',
        'presentation_layer_category' =>  'jira cloud',
        'go_url_path' => 'browse',
        'delete_events' => 'jira:issue_deleted',
        'close_events' => 'done'
    }

    packet_map2 = {
        'packet_type' => 'jira_cloud',
        'group_id' => '2',
        'jira_cloud' =>  {
            'id' => 'issue.key',
            'summary' => 'issue.fields.summary',
            'description' => 'issue.fields.description',
            'status_id' => 'issue.fields.status.id',
            'status_name' => 'issue.fields.status.name',
            'status_category' => 'issue.fields.status.statusCategory.key',
            'assigned_to' => 'issue.fields.assignee',
            'priority' => 'issue.fields.priority.id',
            'tracker_url' => 'issue.fields.issuetype.iconUrl',
            'tracker_key' => 'issue.fields.project.key',
            'tracker_name' => 'issue.fields.issuetype.name',
            'remote_id' => 'issue.fields.project.id',
            'creation_time' => 'issue.fields.created',
            'created_by' => 'issue.fields.creator.name',
            'updated_by' => 'user.name',
            'go_url' => 'issue.fields.project.self',
            'event_name' => 'webhookEvent'
        },
        'description' => 'basic jira cloud mapper',
        'presentation_layer_category' =>  'jira cloud',
        'go_url_path' => 'browse',
        'delete_events' => 'jira:issue_deleted',
        'close_events' => 'done'
    }

    scheduler1 = {
        'pattern_name'  => 'jira_cloud',
        'pattern_id'  => 'jira1',
        'state_object'  => {
            'pattern_id'  => 'jira1',
            'pattern_name'  => 'jira_cloud',
            'view_name'  => 'jiraView',
            'append_always'  => 'true'
        },
        'dml_object'  => {
            'pattern_id'  => 'jira1',
            'view_name'  => 'jiraView',
            'required_attributes'  => 'webhookEvent,issue.key,issue.fields.description,issue.fields.summary,issue.fields.created,issue.fields.priority.id,issue.fields.status.id,issue.fields.assignee.name,issue.fields.project.key,issue.fields.issuetype.name,issue.fields.status.name,issue.fields.project.id,issue.fields.issuetype.iconUrl,issue.fields.project.self,issue.fields.status.statusCategory.key,issue.fields.created,issue.fields.creator.name,user.name',
            'projection'  => 'source_association_key,webhookEvent,issue.key,issue.fields.description,issue.fields.summary,issue.fields.created,issue.fields.priority.id,issue.fields.status.id,issue.fields.assignee.name,issue.fields.project.key,issue.fields.issuetype.name,issue.fields.status.name,issue.fields.project.id,issue.fields.issuetype.iconUrl,issue.fields.project.self,issue.fields.status.statusCategory.key,issue.fields.created,issue.fields.creator.name,user.name',
            'synthetic_dml'  => ['CREATE jiraView'],
            'sql'  => ['CREATE jiraView'],
            'mongodb'  => [ "DataDump.where(:webhookEvent => { '$exists' => true },
                                            :issue.key => { '$exists' => true },
                                            :issue.fields.description => { '$exists' => true },
                                            :issue.fields.summary => { '$exists' => true },
                                            :issue.fields.created => { '$exists' => true },
                                            :issue.fields.priority.id => { '$exists' => true },
                                            :issue.fields.status.id => { '$exists' => true },
                                            :issue.fields.assignee.name => { '$exists' => true },
                                            :issue.fields.project.key => { '$exists' => true },
                                            :issue.fields.issuetype.name => { '$exists' => true },
                                            :issue.fields.status.name => { '$exists' => true },
                                            :issue.fields.project.id => { '$exists' => true },
                                            :issue.fields.issuetype.iconUrl => { '$exists' => true },
                                            :issue.fields.project.self => { '$exists' => true },
                                            :issue.fields.status.statusCategory.key => { '$exists' => true },
                                            :source_association_key => { '$exists' => true },
                                            :webhookEvent => { '$exists' => true },
                                            :issue.key => { '$exists' => true },
                                            :issue.fields.description => { '$exists' => true },
                                            :issue.fields.summary => { '$exists' => true },
                                            :issue.fields.created => { '$exists' => true },
                                            :issue.fields.priority.id => { '$exists' => true },
                                            :issue.fields.status.id => { '$exists' => true },
                                            :issue.fields.assignee.name => { '$exists' => true },
                                            :issue.fields.project.key => { '$exists' => true },
                                            :issue.fields.issuetype.name => { '$exists' => true },
                                            :issue.fields.status.name => { '$exists' => true },
                                            :issue.fields.project.id => { '$exists' => true },
                                            :issue.fields.issuetype.iconUrl => { '$exists' => true },
                                            :issue.fields.project.self => { '$exists' => true },
                                            :issue.fields.created => { '$exists' => true },
                                            :issue.fields.creator.name => { '$exists' => true },
                                            :user.name => { '$exists' => true },

                                            :issue.fields.status.statusCategory.key => { '$exists' => true }).fields(:source_association_key,
                                                                                                                     :webhookEvent,
                                                                                                                     :issue.key,
                                                                                                                     :issue.fields.description,
                                                                                                                     :issue.fields.summary,
                                                                                                                     :issue.fields.created,
                                                                                                                     :issue.fields.priority.id,
                                                                                                                     :issue.fields.status.id,
                                                                                                                     :issue.fields.assignee.name,
                                                                                                                     :issue.fields.project.key,
                                                                                                                     :issue.fields.issuetype.name,
                                                                                                                     :issue.fields.status.name,
                                                                                                                     :issue.fields.project.id,
                                                                                                                     :issue.fields.issuetype.iconUrl,
                                                                                                                     :issue.fields.project.self,
                                                                                                                     :issue.fields.created,
                                                                                                                     :issue.fields.creator.name,
                                                                                                                     :user.name,
                                                                                                                     :issue.fields.status.statusCategory.key)"
            ]
        }
    }

    scheduler2 = {
        'pattern_name'  => 'jira_cloud',
        'pattern_id'  => 'jira2',
        'state_object'  => {
            'pattern_id'  => 'jira2',
            'pattern_name'  => 'jira_cloud',
            'view_name'  => 'jiraView',
            'append_always'  => 'true'
        },
        'dml_object'  => {
            'pattern_id'  => 'jira2',
            'view_name'  => 'jiraView',
            'required_attributes'  => 'webhookEvent,issue.key,issue.fields.description,issue.fields.summary,issue.fields.created,issue.fields.priority.id,issue.fields.status.id,issue.fields.assignee,issue.fields.project.key,issue.fields.issuetype.name,issue.fields.status.name,issue.fields.project.id,issue.fields.issuetype.iconUrl,issue.fields.project.self,issue.fields.status.statusCategory.key,issue.fields.created,issue.fields.creator.name,user.name',
            'projection'  => 'source_association_key,webhookEvent,issue.key,issue.fields.description,issue.fields.summary,issue.fields.created,issue.fields.priority.id,issue.fields.status.id,issue.fields.assignee,issue.fields.project.key,issue.fields.issuetype.name,issue.fields.status.name,issue.fields.project.id,issue.fields.issuetype.iconUrl,issue.fields.project.self,issue.fields.status.statusCategory.key,issue.fields.created,issue.fields.creator.name,user.name',
            'synthetic_dml'  => ['CREATE jiraView'],
            'sql'  => ['CREATE jiraView'],
            'mongodb'  => [ "DataDump.where(:webhookEvent => { '$exists' => true },
                                            :issue.key => { '$exists' => true },
                                            :issue.fields.description => { '$exists' => true },
                                            :issue.fields.summary => { '$exists' => true },
                                            :issue.fields.created => { '$exists' => true },
                                            :issue.fields.priority.id => { '$exists' => true },
                                            :issue.fields.status.id => { '$exists' => true },
                                            :issue.fields.assignee => { '$exists' => true },
                                            :issue.fields.project.key => { '$exists' => true },
                                            :issue.fields.issuetype.name => { '$exists' => true },
                                            :issue.fields.status.name => { '$exists' => true },
                                            :issue.fields.project.id => { '$exists' => true },
                                            :issue.fields.issuetype.iconUrl => { '$exists' => true },
                                            :issue.fields.project.self => { '$exists' => true },
                                            :issue.fields.status.statusCategory.key => { '$exists' => true },
                                            :source_association_key => { '$exists' => true },
                                            :webhookEvent => { '$exists' => true },
                                            :issue.key => { '$exists' => true },
                                            :issue.fields.description => { '$exists' => true },
                                            :issue.fields.summary => { '$exists' => true },
                                            :issue.fields.created => { '$exists' => true },
                                            :issue.fields.priority.id => { '$exists' => true },
                                            :issue.fields.status.id => { '$exists' => true },
                                            :issue.fields.assignee => { '$exists' => true },
                                            :issue.fields.project.key => { '$exists' => true },
                                            :issue.fields.issuetype.name => { '$exists' => true },
                                            :issue.fields.status.name => { '$exists' => true },
                                            :issue.fields.project.id => { '$exists' => true },
                                            :issue.fields.issuetype.iconUrl => { '$exists' => true },
                                            :issue.fields.project.self => { '$exists' => true },
                                            :issue.fields.created => { '$exists' => true },
                                            :issue.fields.creator.name => { '$exists' => true },
                                            :user.name => { '$exists' => true },
                                            :issue.fields.status.statusCategory.key => { '$exists' => true }).fields(:source_association_key,
                                                                                                                     :webhookEvent,
                                                                                                                     :issue.key,
                                                                                                                     :issue.fields.description,
                                                                                                                     :issue.fields.summary,
                                                                                                                     :issue.fields.created,
                                                                                                                     :issue.fields.priority.id,
                                                                                                                     :issue.fields.status.id,
                                                                                                                     :issue.fields.assignee,
                                                                                                                     :issue.fields.project.key,
                                                                                                                     :issue.fields.issuetype.name,
                                                                                                                     :issue.fields.status.name,
                                                                                                                     :issue.fields.project.id,
                                                                                                                     :issue.fields.issuetype.iconUrl,
                                                                                                                     :issue.fields.project.self,
                                                                                                                     :issue.fields.created,
                                                                                                                     :issue.fields.creator.name,
                                                                                                                     :user.name,
                                                                                                                     :issue.fields.status.statusCategory.key)"
            ]
        }
    }

    db[:patterns].drop()
    db[:schedulers].drop()
    db[:packet_maps].drop()
    [pattern1,pattern2].each { | p_hash | Pattern.create(p_hash)}
    [scheduler1,scheduler2].each { | s_hash | Scheduler.create(s_hash)}
    [packet_map1,packet_map2].each { | m_hash | PacketMaps.create(m_hash) }
  end

end
