#
# Copyright 2016 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#
require 'migrations/exceptions'
class UpdatePermissionRabbitmqUsers < Migration

  extend MigrationHelper

  def self.up
    rabbit_config ||= Rails.configuration.local_settings.load('rabbitmq')
    queue_server = Rabbit::Management.new(rabbit_config['admin_user'], rabbit_config['admin_password'], rabbit_config['uri'])
    rabbit_users = queue_server.users

    # set new permission
    count = 0
    rabbit_users.map do | rabbit_user |
      count += 1

      if [APP_NAME, OLD_APP_BASE_QUEUE_NAME].include?(rabbit_user['tags'])
        # Set new permissions
        queue_server.set_permissions(rabbit_user['name'])
      end

      # Set new permissions for a specific user - orchestrate / eventq
      if %w(eventq orchestrate).include?(rabbit_user['name'])
        common_permission = "^(#{OLD_APP_BASE_QUEUE_NAME}|#{APP_BASE_QUEUE_NAME}).*|amq.default|#{EVENT_HUB_NAME}"
        rabbit_admin_permission = { configure: "#{common_permission}", write: "#{common_permission}", read: "#{common_permission}" }
        queue_server.set_permissions(rabbit_user['name'], rabbit_admin_permission)
      end

      # puts "#{count} / #{rabbit_users.count}"
    end
  end

  def self.down
    rabbit_permissions = { configure: '^orchestrate.*|amq.default', write: '^orchestrate.*|amq.default', read: ''}
    rabbit_admin_permissions = { configure: '^orchestrate.*', write: '^orchestrate.*|amq.default', read: '^orchestrate.*|amq.default'}
    rabbit_config ||= Rails.configuration.local_settings.load('rabbitmq')
    queue_server = Rabbit::Management.new(rabbit_config['admin_user'], rabbit_config['admin_password'], rabbit_config['uri'])
    rabbit_users = queue_server.users

    # reset with old permission
    count = 0
    rabbit_users.map do | rabbit_user |
      count += 1
      if [APP_NAME, OLD_APP_BASE_QUEUE_NAME].include?(rabbit_user['tags'])
        # Reset permission with old
        queue_server.set_permissions(rabbit_user['name'], rabbit_permissions)
      end

      # Set new permissions for a specific user - orchestrate / eventq
      queue_server.set_permissions(rabbit_user['name'], rabbit_admin_permissions) if %w(eventq orchestrate).include?(rabbit_user['name'])

      # puts "#{count} / #{rabbit_users.count}"
    end
  end
end