#
# Copyright 2014 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

require 'application_constants'

class CustomQueueItemParser

  QUEUE_NAME = CUSTOM_QUEUE_NAME 
  DATA_SEGMENT = [:custom_schema, :custom_data]

  # param [String] JSON message from Rabbit
  def self.process_message(message)
    message_hash = JSON.parse(message).with_indifferent_access
    type = DATA_SEGMENT.find {|key| message_hash.include? key }

    unless type
      LogUtils.log_error_and_info(I18n.t('queue_parser.no_recognized_data'), message_hash)
      return
    end

    raise MissingFieldError.new(:api_version) if message_hash[:api_version].nil?
    raise MessageProcessingError.new(I18n.t('api.invalid.version')) if message_hash[:api_version] != '1'

    import_message(type, message_hash)
  rescue MissingFieldError => mfe
    LogUtils.log_error_and_info(I18n.t('api.required_field', field_name: mfe.message.to_s), message_hash)
  rescue NoSuchSourceAssociationKey => nssak
    LogUtils.log_error_and_info(I18n.t('custom_activity_source.invalid.source_not_found', source_association_key: nssak.message), message_hash)
  rescue JSON::ParserError
    LogUtils.log_error_and_info(I18n.t('api.json_required'), message)
  rescue InconsistentStateError => e
    LogUtils.log_error_and_info("#{I18n.t('api.errors_found')} #{e.message}", message_hash)
  rescue MessageProcessingError => e
    LogUtils.log_error_and_info("#{I18n.t('api.errors_found')} #{e.message}", message_hash)
  rescue ProjectNotActive => e
    LogUtils.log_error_and_info(e.message, message_hash)
  rescue SourceNotActive => e
    LogUtils.log_error_and_info(I18n.t('api.source_deactivated', attribute: e.message), message_hash)
  end

  def self.import_message(message_type, message_hash)
    sak = message_hash[:source_association_key]
    if message_type == DATA_SEGMENT[1] # Custom_data
      raise MissingFieldError.new(:source_association_key) if sak.nil?
      source = find_source(sak)
      source.handle_message(message_hash[message_type])
    elsif message_type == DATA_SEGMENT[0] # Custom_schema
      CustomSchemaStore.build_schema(message_hash[message_type]).tap do |schema_store|
        if sak.present?
          source = find_source(sak)
          source.associate_with_schema_store(schema_store)
        end
      end
    end
  end

  def self.find_source(sak)
    source = CustomActivitySource.find_by_source_association_key(sak)
    raise NoSuchSourceAssociationKey.new(sak) unless source
    raise SourceNotActive.new(sak) unless source.active
    source
  end

  class << self
    include ::NewRelic::Agent::Instrumentation::ControllerInstrumentation
    add_transaction_tracer :process_message, category: :task, name: self.to_s
  end
end
