#
# Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

require 'net/http'
require 'net/https'

class InvalidURI < StandardError
end

# Creates HTTP(S) connections to servers
class HttpConfiguration

  # Creates a http(s) connection to the server
  # @param [URI, String] server_info the server URI
  # @return [Net::HTTP] the connection
  def self.create_connection(server_info)
    server_info = URI(server_info) if server_info.is_a?(String)
    raise URI::InvalidURIError unless server_info.scheme

    Net::HTTP.new(server_info.host, server_info.port).tap do |http|
      use_ssl = (server_info.scheme == 'https')
      http.use_ssl = use_ssl
      http.ca_file = 'config/cacert.pem' if use_ssl
      http.verify_mode = OpenSSL::SSL::VERIFY_NONE
      #http.set_debug_output $stdout #uncomment to see http info in logs
    end
  end


  # Creates a GET request for the uri
  # @param [String] uri the server URI
  # @param [Hash] options contains any of the following:
  #  :username - the username
  #  :password - the password
  def self.create_get_request(uri, options = nil)
    Net::HTTP::Get.new(uri).tap  do |request|
      request.basic_auth options[:username], options[:password] unless options[:username].blank?
    end
  end
end
