#
# Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

class UnableToContactServerError < StandardError
end
class MalformedUrlError < StandardError
end
class LocationError < StandardError
end
class AuthenticationError < StandardError
end


require 'addressable/uri'

# Encapsulates methods for testing HTTP connections
class HttpConnectionValidator < ActiveModel::Validator
  # Creates a new [HttpConnectionValidator].
  # @param [Hash] options the options for this validator. This particular one needs the :attribute value set with the
  #   symbol of the attribute to validate.
  def initialize(options)
    super
    @attribute = options[:attribute]
  end

  # Validates the attribute given in constructor. Adds errors to the model's errors list. Allows empty URLs
  # @param record the model to validate
  def validate(record)
    if record.send(@attribute).blank?
      record.errors.add(@attribute, I18n.t('errors.http.missing_uri'))
      return
    end

    begin
      validate_url!(record)
    rescue UnableToContactServerError
      record.errors.add(@attribute, I18n.t('errors.http.unreachable_uri'))
    rescue MalformedUrlError
      record.errors.add(@attribute, I18n.t('errors.http.malformed_uri'))
    rescue LocationError #wonthappen
      record.errors.add(@attribute, I18n.t('errors.http.page_not_found'))
    rescue AuthenticationError #wonthappen
      record.errors.add(@attribute, I18n.t('errors.http.requires_authentication'))
    end
  end

  private

  # Tries to connect to the provided URL.
  # @param record the record that has the uri to test
  # @return [Boolean] True if the connection succeeds, false otherwise.
  def validate_url!(record)
    url = record.send(@attribute)
    begin
      http = HttpConfiguration.create_connection(url)
    rescue URI::InvalidURIError
      raise MalformedUrlError.new(url)
    end
  end
end
