#
# Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

require 'erb'

# Generates a newrelic.yml configuration file for a given environment.  Keys have to be set by the caller.
class NewRelicMaker
  cattr_accessor :keys

  # @param [Pathname|IO] path_or_stream An object that responds to #read and returns an ERB template string.
  def initialize(path_or_stream)
    @template_source = path_or_stream.read
    @template = ERB.new(@template_source)
  end

  # Render a NewRelic template out as a newrelic.yml configuration file.
  #
  # @param [String] path The output path to write the newrelic.yml file.
  # @param [Symbol] env The environment (:staging, :production, etc.) to find the NewRelic registration key.
  def generate(path, env)
    self.class.keys ||= {}
    raise ArgumentError.new("Bad environment #{env}.") unless keys.keys.include? env.to_sym
    open(path, 'w') { |file| write_config(file, env) }
  end

  # Look up the key and write out the configuration settings to a stream.
  #
  # @param [IO] file An output stream to write the resultant configuration settings to.
  # @param [Symbol] env The environment (:staging, :production, etc.) to find the NewRelic registration key.
  def write_config(file, env)
    key = ENV['NEWRELIC_KEY'] || keys[env.to_sym]
    custom_name = 'staging' if env.to_sym == :staging
    file.write(@template.result(binding))
  end
end
