#
# Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

# Functions related to the Opaque Activity URLs
# see https://forge.collab.net/sf/wiki/do/viewPage/projects.orch/wiki/ShortURLs for commentary

class OpaqueID
  # Pseudorandom dictionary. Unlikely to cause embarrassing URLs or trivial increments (URL + 1)
  ALPHABET = ["k", "d", "P", "c", "G", "F", "5", "f", "m", "n", "2", "7", "V", "K", "6", "4", "g", "B", "W", "8", "D",
              "L", "l", "b", "Z", "T", "3", "C", "w", "h", "H", "R", "z", "j", "r", "J", "v", "9", "N", "x", "p", "t",
              "X", "M", "q", "s", "Q", "S"]
# Originally from:
#   ALPHABET = (('a'..'z').to_a + ('A'..'Z').to_a + ('2'..'9').to_a - 'aeiouyAEIOUY'.chars.to_a).shuffle(random: Random.new(42))
# Converted to inline because the random sequence isn't the same on MRI
  RADIX = ALPHABET.length

  # Convert a number into a 'hash' for an opaque ID
  # (Not strictly a hash since collisions are bad news.)
  # @param a positive integer
  # @return a URL-safe string of pseudo-gibberish

  def self.convert(number)
    answer = []
    while number > 0
      # Putting the least significant digits first makes it easier to tell 2 IDs apart
      # and makes the IDs seem more random
      number, remainder = number.divmod(RADIX)
      answer << ALPHABET[remainder]
    end
    answer.join
  end
end
