#
# Copyright 2015 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

require File.expand_path('../../../lib/installer/helpers.rb',  __FILE__)

Rake::Task['assets:precompile'].enhance do
  Rake::Task["#{APP_NAME}:create_non_digest_icons"].invoke
end

namespace APP_NAME.to_sym do
  desc 'Update IAF configuration'
  task :update_iaf_configuration => :environment do
    CtfConfigurer.new.edit_iaf_application
  end

  desc 'Copy non-digest version of icon images to public directory'
  task :create_non_digest_icons => :environment do

    # copy files to public (all icons, not just custom sources)
    source_directory = File.join(Rails.root, 'app/assets/images/icons')
    destination_directory = File.join(Rails.root, 'public/assets/icons')
    FileUtils.cp_r(source_directory + '/.', destination_directory)

    # point rails to copied files in the manifest file (this yaml file is used to map
    # asset urls in templates/css to the precompiled versions of the assets)
    manifest_path = Dir.glob(File.join(Rails.root, 'public/assets/manifest-*.json')).first
    manifest_data = JSON.load(File.new(manifest_path))
    manifest_data['assets'].each do |key, value|
      if key =~ /icons\/custom_source\/.*/
        manifest_data['assets'][key] = key
      end
    end
    File.open(manifest_path, 'w') do |file|
      file.puts manifest_data.to_json
    end
  end
end
