#
# Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

class BlankUriError < StandardError
end
class InvalidURIError < StandardError
end

# Utility module for parsing/splitting URIs
class UriUtils
  # Parses the server's URI
  # @return [URI] Parsed URI.
  def initialize(uri)
    raise BlankUriError.new(uri) if uri.blank?
    @raw_uri = uri
    @uri = URI(URI.encode(uri))
  end

  # @return [Hash] the scheme, host and port of the URI, in a hash
  def host_parts
    {
        scheme: @uri.scheme,
        host: @uri.host,
        port: @uri.port
    }
  end

  def host
    @uri.host
  end

  def port
    @uri.port.to_i
  end

  def path
    @uri.path
  end

  def ssl?
    @uri.scheme == 'https'
  end

  # Splits the URI out into its component parts
  # @return [Hash] Contains the path, address and the original URI, urlencoded
  def split_uri
    encoded = URI.encode(@raw_uri)

    begin
      path = URI(encoded).request_uri
    rescue NoMethodError
      raise InvalidURIError.new()
    end

    full_uri = URI(encoded)
    full_uri.path = ''
    address = full_uri.to_s

    {path: path, address: address, encoded_uri: encoded}
  end
end