#!/opt/collabnet/ruby/bin/ruby
#
# Copyright 2016 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

require 'yaml'
require 'uri'
require 'fileutils'
require 'getoptlong'
require 'securerandom'
require File.expand_path("../../lib/application_constants", __FILE__)
require File.expand_path("../../lib/rabbit/management", __FILE__)


STDOUT.sync = true
STDIN.sync = true

PERMISSION_QUEUE_USERNAME = 'permission_publisher'
PERMISSION_QUEUE_PASSWORD = 'permission_publisher_password'
GENERATED_PASSWORD_LENGTH = 32
RABBITMQ_HOME_DIR = File.join(COLLABNET_DIR, 'rabbitmq')
RABBITMQ_PASSWORDS_FILE = File.join(RABBITMQ_HOME_DIR, '.rabbit_passwords')

def update_rabbit_host

  data = YAML.load(File.read(APP_DEFAULTS_YAML_FILE))
  uri = URI(data['rabbitmq']['production']['uri'])
  loopback = ['127.0.0.1', 'localhost']

  if loopback.include?(uri.host)
    puts "ERROR: the value for the RabbitMQ host is a loopback interface."
    while loopback.include?(uri.host)
      print ">> Please supply the FQDN of the rabbit host: "
      host = STDIN.gets.strip
      uri.host = host unless host.empty?
    end
    data['rabbitmq']['production']['uri'] = uri.to_s
    File.open(defaults_file, 'w') { |f|
      f.write data.to_yaml
    }
    puts "#{defaults_file} successfully updated."
  end

end


def create_rabbit_publisher_user

  existing_yaml = YAML::load_file(APP_DEFAULTS_YAML_FILE)
  rabbitmq_config = existing_yaml["rabbitmq"]['production']

  if rabbitmq_config['permission_publisher'].nil? && rabbitmq_config['permission_publisher_password'].nil?

    # # create a permission publisher in rabbitmq
    permission_queue_password = SecureRandom.hex(GENERATED_PASSWORD_LENGTH).to_s
    common_permission = 'eventq.ctf_user_permissions|event.hub'
    specific_permissions = { configure: "#{common_permission}", write: "#{common_permission}", read: "#{common_permission}" }

    queue_server = Rabbit::Management.new(rabbitmq_config['admin_user'], rabbitmq_config['admin_password'], rabbitmq_config['uri'])
    queue_server.create_user(PERMISSION_QUEUE_USERNAME, permission_queue_password) && queue_server.set_permissions(PERMISSION_QUEUE_USERNAME, specific_permissions)

    # Due to different file setup in local dev environment, manual steps are required
    # to update defaults.yml or user/username.yml file

    existing_yaml['rabbitmq']['production'][PERMISSION_QUEUE_USERNAME] = PERMISSION_QUEUE_USERNAME
    existing_yaml['rabbitmq']['production'][PERMISSION_QUEUE_PASSWORD] = permission_queue_password

    # update defaults.yml file with latest values
    File.open(APP_DEFAULTS_YAML_FILE, 'w') { |f| f.write existing_yaml.to_yaml}

    update_rabbit_file(PERMISSION_QUEUE_USERNAME, permission_queue_password)
  else
    puts "permission_publisher user already exists in #{RABBITMQ_PASSWORDS_FILE} file."
  end

end

# update rabbit_passwords file
def update_rabbit_file(user, password)
  open(RABBITMQ_PASSWORDS_FILE, 'a') { |f|
    f.puts "#{user}: #{password}"
  }
end

def usage(message=nil)
  scriptname = File.basename(__FILE__)
  puts "\nERROR: #{message}\n\n" if message
  puts "Overview:"
  puts "  Update rabbit host OR / AND"
  puts "  create permission_publisher_user"
  puts
  puts "  *** USE THIS TOOL WITH EXTREME CAUTION ***"
  puts
  puts "Usage:"
  puts "  #{scriptname} --update_rabbit_host"
  puts "  #{scriptname} --update_rabbit_host --create_rabbit_publisher_user"
  puts
  puts "Options:"
  puts " -u, --update_rabbit_host    Update RabbitMQ host"
  puts " -c, --create_rabbit_publisher_user         Create new RabbitMQ user (permission_publisher)"
  puts " -h, --help        Show this message and exit"
  puts
  exit
end

begin

  unless File.file? APP_DEFAULTS_YAML_FILE
    puts "\nFATAL: no such file #{APP_DEFAULTS_YAML_FILE}\n\n"
    exit 1
  end

  getopts = GetoptLong.new(
      ['-h', '--help', GetoptLong::NO_ARGUMENT],
      ['-u', '--update_rabbit_host', GetoptLong::NO_ARGUMENT],
      ['-c', '--create_rabbit_publisher_user', GetoptLong::NO_ARGUMENT],
  )

  getopts.each do |opt, arg|
    case opt
      when '-h'
        usage
      when '-u'
        update_rabbit_host
      when '-c'
        create_rabbit_publisher_user
    end
  end
end
