#
# Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

module CtfAssociationHelper

  # Creates a link to the [CtfAssociation] based on the given CTF app URL.
  # @param [String] teamforge_app_url the URL to the CTF application (including /sf)
  # @param [CtfAssociation] ctf_association the association to link
  # @param [Hash] opts the element options for content_tag, when a block is given
  # @param [Block] block optionally provides the content of the <a> tag
  # @return the HTML anchor tag with the link
  def link_to_association(ctf_association, opts = {}, &block)
    url = go_url(ctf_association)
    if block_given?
      opts[:href] = url
      content_tag(:a, opts, &block)
    else
      link_to ctf_association.item_id, url, class: "external-link"
    end
  end


  # Creates the URL for a commit's CTF association.
  # @param [CtfAssociation] ctf_association the association to create the url for
  # @return the URL for the association
  def go_url(ctf_association)
    URI.join(Rails.configuration.teamforge["server"], "/sf/go/", ctf_association.item_id).to_s
  end

end