#
# Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

class CtfRepository < CommitServer
  key :ctf_id, String # For CTF

  validates_presence_of :ctf_id
  validates :repository_uri, :uniqueness => {:scope => [:project_id]},  allow_blank: false

  def self.permitted_attrs
    super.concat(["ctf_id"])
  end

  # since it appears that there is no equivalent to ActiveRecord::Base#becomes in MongoMapper
  # we use this solution to make forms work for SCI cases (as an alternative to explicit  routing)
  def self.model_name
    CommitServer.model_name
  end

  # List of information on repositories available for the given project
  # Includes repositories that may already be in use
  #
  # @see ScmService#repositories_for_ctf_project
  # @return [[Hash]] hash of name, url and ctf_id for this project
  def self.project_repositories(ctf, project)
    ctf_project_id = project.project_configuration.ctf_project_id
    ScmService.new(ctf).repositories_for_ctf_project(ctf_project_id)
  rescue NoMethodError
    Rails.logger.error("Was not able to find a Teamforge project: #{project.inspect}")
    []
  end

end
