#
# Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

class CtfAssociationJob
  extend ApplicationHelper
  @queue = :ctf_association_job

  # Resque job to publish associations in CTF for Commits and Work Items originating in CTF.
  # @param [BSONId] activity_summary_id the activity summary whose associations should be published
  def self.perform(activity_summary_id)
    logfile = ENV.delete('LOGFILE')

    if logfile.present?
      Rails.logger = ActiveSupport::Logger.new(logfile)
    end

    activity_summary = ActivitySummary.find_by_id(activity_summary_id)
    unless activity_summary
      Rails.logger.warn "ActivitySummary ID not found: #{activity_summary_id}"
      return
    end

    Rails.logger.debug " --- Performing CtfAssociationJob for activity_summary: #{activity_display_name(activity_summary)}"

    related_activities, ctf_ids = self.association_endpoints(activity_summary)

    Rails.logger.debug " --- Activity Summary has these CTF associations: #{ctf_ids}"

    activity_names = related_activities.map { |activity| activity_display_name(activity) }
    Rails.logger.debug " --- Associating these related_activities: #{activity_names}"

    related_activities.each do |activity|
      activity.associate_to_ctf_item_array(ctf_ids)
    end


    Rails.logger.debug " --- Done performing CtfAssociationJob for activity_summary.remote_id: #{activity_display_name(activity_summary)}"
  end

  # returns the endpoints for relashionship. We'll use the cartesian product of those when making SOAP calls to create
  # an associaton inside TeamForge
  def self.association_endpoints(activity_summary)
    from = []
    to = []
    case activity_summary
      when CommitSummary
        if activity_summary.ctf_item
          from = activity_summary.all_activity_summary_associations(false)
          to = (activity_summary.associated_work_items + [activity_summary.ctf_item]).map(&:ctf_id)
        end
      when WorkItemSummary
        from = activity_summary.activity_summaries.reject {|x| x.is_a?(CommitSummary) && x.ctf_item.present? } # so that we don't double report the association
        to = [activity_summary.remote_id]
    end
    [from, to]
  end
end
