#
# Copyright 2016 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

# Helper module for finding and listing the IAF association prefixes

class CtfPrefixNotFoundError < StandardError; end  
  
module CtfAssociationPrefixes
  # activities are instead published with specified prefix
  OVERRIDE_PREFIX = Rails.configuration.teamforge[:iaf].try(:[], :prefix)

  # List of IAF prefixes for the application 
  # @return [Array[String]] of all prefixes
  def ctf_association_prefixes
    OVERRIDE_PREFIX ? [OVERRIDE_PREFIX] : CtfDefaultPrefixes::PREFIXES.values
  end

  # Find the IAF prefix which should be used by a given Activity class instance
  # Raises error if class is not mapped to any prefix
  # @param [Activity] klass whose prefix to find
  # @return [String] the prefix
  def class_to_ctf_association_prefix(klass)
    OVERRIDE_PREFIX || CtfDefaultPrefixes::PREFIXES[klass.to_s.sub("Summary","").underscore.to_sym] || raise(CtfPrefixNotFoundError)
  end
end
