#
# Copyright 2013 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

# Mix into Activity subclasses which should be associated to CTF items
module CtfAssociationPublisher
  include CtfAssociationPrefixes

  # Ruby language hook method, called when module is included into Activity
  def self.included(base)
    # persist the ctf ids to which this item is associated
    base.key :associated_ctf_ids, Array
  end

  # Associate the including Activity to a TeamForge item
  # but only once
  # @param [String] ctf_id (artf1736272, wiki10392 etc.)
  def associate_to_ctf_item(ctf_id)
    unless self.associated_ctf_ids.include?(ctf_id)
      session = CtfService.create_authenticated_teamforge_session
      session.teamforge.create_association_with_titles(origin_id: ctf_id, origin_title: nil,
                                           target_id: "#{class_to_ctf_association_prefix(self.class)}_#{self.opaque_id}",
                                           target_title: self.ctf_description,
                                           description: self.ctf_description)
      self.associated_ctf_ids << ctf_id
      self.save
    end
  end

  # Associate the including Activity to an array of ctf ids. This method
  # will try all elements of the array, even if one or more items fails (warning logged)
  # @param [Array] ctf_ids
  def associate_to_ctf_item_array(ctf_ids)
    ctf_ids.each do |ctf_id|
      begin
        associate_to_ctf_item(ctf_id)
      rescue Exception => e
        logger.warn("Unable to publish association with CTF item '#{ctf_id}': #{e.message}")
      end
    end
  end

  # string that gets displayed in CTF
  def ctf_description
    "#{ctf_activity_type_name}: #{display_number} - #{source.display_name}"
  end
  
  # activity type name that gets displayed in CTF
  def ctf_activity_type_name
    self.class.name.sub("Summary", "").camelize
  end
  
  # sends the commit ids that are associated to this activity into a job queue for passing to CTF
  def publish_associations_to_ctf
    associated_commit_summaries.select(&:settled).each do |commit_summary|
      Resque.enqueue(CtfAssociationJob, commit_summary.id)
    end
  rescue
    Rails.logger.error "Unable to create item associations in TeamForge using #{Resque.inspect}"
    raise
  end
end