#
# Copyright 2016 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

require 'application_constants'

# Parses ctf user permission event messages
class CtfUserPermissionQueueItemParser
  include CtfUserAuthorization

  QUEUE_NAME = CTFUSERPERMISSION_QUEUE_NAME
  DATA_SEGMENT = [:ctf_user_permission]
  EVENTQ_DATA_SEGMENT = [:eventq_permission]

  # Creates a new instance of the class and parses the review JSON
  # @param [String] message the JSON message from Rabbit
  def self.process_message(message)
    message_hash = JSON.parse(message).with_indifferent_access
    message_type = DATA_SEGMENT.find { |key| message_hash.include? key }
    eventq_message = EVENTQ_DATA_SEGMENT.find { |key| message_hash[message_type].include? key } if message_type

    unless eventq_message
      LogUtils.log_error_and_info(I18n.t('queue_parser.no_recognized_data'), message_hash)
      return
    end

    CtfUserPermissionQueueItemParser.handle_message(message_hash[message_type][eventq_message])

  rescue MissingFieldError => mfe
    LogUtils.log_error_and_info(I18n.t('api.required_field', field_name: mfe.message.to_s), message_hash)
  rescue JSON::ParserError
    LogUtils.log_error_and_info(I18n.t('api.json_required'), message)
  end

  class << self
    include ::NewRelic::Agent::Instrumentation::ControllerInstrumentation
    add_transaction_tracer :process_message, category: :task, name: self.to_s
  end

end
